// LeftView.cpp : implementation of the CLeftView class
//

#include "stdafx.h"
#include "TESPCD.h"

#include "TESPCDDoc.h"
#include "LeftView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLeftView

IMPLEMENT_DYNCREATE(CLeftView, CTreeView)

BEGIN_MESSAGE_MAP(CLeftView, CTreeView)
	//{{AFX_MSG_MAP(CLeftView)
	ON_COMMAND(ID_CHECKTYPE_FULLCHECK, OnChecktypeFullcheck)
	ON_COMMAND(ID_CHECKTYPE_SINGLEVERSUS, OnChecktypeSingleversus)
	ON_UPDATE_COMMAND_UI(ID_CHECKTYPE_FULLCHECK, OnUpdateChecktypeFullcheck)
	ON_UPDATE_COMMAND_UI(ID_CHECKTYPE_SINGLEVERSUS, OnUpdateChecktypeSingleversus)
	ON_WM_CONTEXTMENU()
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
	//}}AFX_MSG_MAP
	// Standard printing commands
	//ON_COMMAND(ID_FILE_PRINT, CTreeView::OnFilePrint)
	//ON_COMMAND(ID_FILE_PRINT_DIRECT, CTreeView::OnFilePrint)
	//ON_COMMAND(ID_FILE_PRINT_PREVIEW, CTreeView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLeftView construction/destruction

CLeftView::CLeftView()
{
	m_bFileRootSet = false;
}

CLeftView::~CLeftView()
{
}

BOOL CLeftView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= TVS_HASLINES | TVS_LINESATROOT | TVS_HASBUTTONS;

	return CTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CLeftView drawing

void CLeftView::OnDraw(CDC* pDC)
{
	CTESPCDDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}


/////////////////////////////////////////////////////////////////////////////
// CLeftView printing

void CLeftView::OnInitialUpdate()
{
	CTreeView::OnInitialUpdate();

	HICON hKnife = AfxGetApp()->LoadIcon(IDR_KNIFE);
	HICON hMorr = AfxGetApp()->LoadIcon(IDR_MORR);

	CTreeCtrl* pTree= &GetTreeCtrl();

	m_ImgList.Create(16,16,ILC_COLOR,0,1);
	m_ImgList.Add(hKnife);
	m_ImgList.Add(hMorr);
		
	pTree->SetImageList(&m_ImgList,TVSIL_NORMAL);

	if(!m_bFileRootSet)
	{
		m_bFileRootSet = true;
		CTESPCDDoc* pDoc = GetDocument();
		pDoc->m_FilesRoot = pTree->InsertItem("Plugin Files",1,1);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CLeftView diagnostics

#ifdef _DEBUG
void CLeftView::AssertValid() const
{
	CTreeView::AssertValid();
}

void CLeftView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}

CTESPCDDoc* CLeftView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTESPCDDoc)));
	return (CTESPCDDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLeftView message handlers

void CLeftView::OnChecktypeFullcheck() 
{
	// TODO: Add your command handler code here
	CTreeCtrl* pTree= &GetTreeCtrl();
	CTESPCDDoc* pDoc = GetDocument();
	if(pDoc->IsWorkerThreadActive())
		return;

	pDoc->m_nOperationType = pDoc->FULLCHECK;
	if(pDoc->m_SingleMaster)
	{
		if(pTree->DeleteItem(pDoc->m_SingleMaster))
			pDoc->m_SingleMaster = NULL;
	}
}

void CLeftView::OnChecktypeSingleversus() 
{
	// TODO: Add your command handler code here
	CTreeCtrl* pTree= &GetTreeCtrl();
	CTESPCDDoc* pDoc = GetDocument();
	if(pDoc->IsWorkerThreadActive())
		return;

	pDoc->m_nOperationType = pDoc->SINGLEVERSUS;
	if(!pDoc->m_SingleMaster)
		pDoc->m_SingleMaster = pTree->InsertItem("Master Plugin",1,1,TVI_ROOT, TVI_FIRST);
	
}

void CLeftView::OnUpdateChecktypeFullcheck(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	CTESPCDDoc* pDoc = GetDocument();
	if(pDoc->IsWorkerThreadActive())
		return;

	if(pDoc->m_nOperationType == pDoc->FULLCHECK)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);
	
}

void CLeftView::OnUpdateChecktypeSingleversus(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
	CTESPCDDoc* pDoc = GetDocument();
	if(pDoc->IsWorkerThreadActive())
		return;

	if(pDoc->m_nOperationType == pDoc->SINGLEVERSUS)
		pCmdUI->SetCheck(1);
	else
		pCmdUI->SetCheck(0);

}

void CLeftView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if ( pWnd != (CWnd*) &GetTreeCtrl())
	{
		CWnd::OnContextMenu(pWnd, point);//CDialog::OnContextMenu(pWnd, point);
		return;
	}
	
	CPoint CurPos;
	UINT nFlags;
	CMenu menu; 
	CMenu* pContextMenu;
	CTreeCtrl* pTreeCtrl = &GetTreeCtrl();

	// Handle Shift-F10
	if (point.x == -1 && point.y == -1)
		point = (CPoint) GetMessagePos();
	
	GetCursorPos(&CurPos);
	pTreeCtrl->ScreenToClient(&CurPos);
	
	HTREEITEM hItemClk = pTreeCtrl->HitTest(CurPos, &nFlags);
	if(hItemClk != NULL)
	{
		pTreeCtrl->SelectItem( hItemClk);
	}

	menu.LoadMenu(IDR_CONTEXTTREE);

	pContextMenu = menu.GetSubMenu(0);
	pContextMenu->TrackPopupMenu(TPM_LEFTALIGN|TPM_RIGHTBUTTON, point.x, point.y, this, 0);

}

void CLeftView::OnRclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// TODO: Add your control notification handler code here
	SendMessage(WM_CONTEXTMENU, (WPARAM) m_hWnd, GetMessagePos());

	*pResult = 0;
}
