// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "TESPCD.h"

#include "ProgressBar.h"
#include "MainFrm.h"

#include "LeftView.h"
#include "ConflictView.h"
#include "WarningView.h"
#include "EditorView.h"

#include "TESPCDDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_SYSCOMMAND()
	ON_COMMAND(ID_APP_EXIT, OnAppExit)
	ON_COMMAND(ID_EDTPANE, OnEdtpane)
	ON_COMMAND(ID_CNFPANE, OnCnfpane)
	ON_COMMAND(ID_WRNPANE, OnWrnpane)
	ON_WM_SHOWWINDOW()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_cSplitters = sizeof( m_pWndSplitter ) / sizeof( m_pWndSplitter[0]);
	m_bfPanelsShown = 0xFF;
	m_bInitShow = TRUE;
}

CMainFrame::~CMainFrame()
{
	for(int i = 0;i < m_cSplitters;i++)
		delete m_pWndSplitter[i];
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.CreateEx(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}
	/*if (!m_wndDlgBar.Create(this, IDR_MAINFRAME, 
		CBRS_ALIGN_TOP, AFX_IDW_DIALOGBAR))
	{
		TRACE0("Failed to create dialogbar\n");
		return -1;		// fail to create
	}*/

	if (!m_wndReBar.Create(this) ||
		!m_wndReBar.AddBar(&m_wndToolBar) /*||
		!m_wndReBar.AddBar(&m_wndDlgBar)*/)
	{
		TRACE0("Failed to create rebar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY);

	return 0;
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT /*lpcs*/,
	CCreateContext* pContext)
{

	m_pWndSplitter[0] = new CSplitterWndEx();

	// create initial splitter window
	if (!m_pWndSplitter[0]->Create(this,NULL,NULL,pContext,TRUE))
		return FALSE;
	
	// split first splitter in two
	m_pWndSplitter[1] = m_pWndSplitter[0]->AddSubDivision(RIGHT_SIDE,NULL,NULL,pContext,false);
	if(!m_pWndSplitter[1])
		return FALSE;
	
	// split second splitter in two
	m_pWndSplitter[2] = m_pWndSplitter[1]->AddSubDivision(BOTTOM_SIDE,NULL,NULL,pContext,false);
	if(!m_pWndSplitter[2])
		return FALSE;

	if (m_pWndSplitter[0]->AddView(LEFT_SIDE, RUNTIME_CLASS(CLeftView), CSize(200, 100), pContext) == -1 ||
		m_pWndSplitter[1]->AddView(TOP_SIDE, RUNTIME_CLASS(CConflictView), CSize(100, 400), pContext) == -1 ||
		m_pWndSplitter[2]->AddView(TOP_SIDE, RUNTIME_CLASS(CWarningView), CSize(100, 100), pContext) == -1 ||
		m_pWndSplitter[2]->AddView(BOTTOM_SIDE, RUNTIME_CLASS(CEditorView), CSize(100, 100), pContext) == -1) 
	{
		m_pWndSplitter[2]->DestroyWindow();
		m_pWndSplitter[1]->DestroyWindow();
		m_pWndSplitter[0]->DestroyWindow();
		return FALSE;
	}

	return TRUE;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	HWND hwndDesktop, hwndStartMenuBar; 
	RECT rcDesktop, rcStartMenuBar; 
 
    hwndDesktop = ::GetDesktopWindow(); 
    
	if(hwndDesktop != NULL)
	{
        ::GetWindowRect(hwndDesktop, &rcDesktop);
		if((hwndStartMenuBar = ::FindWindowEx(hwndDesktop,NULL,"Shell_TrayWnd","")) != NULL)
		{
			::GetWindowRect(hwndStartMenuBar, &rcStartMenuBar);

			cs.cx = rcDesktop.right - rcDesktop.left;
			cs.cy = rcDesktop.bottom - rcDesktop.top - (rcStartMenuBar.bottom - rcStartMenuBar.top);
		}
		else
		{
			cs.cx = rcDesktop.right - rcDesktop.left;
			cs.cy = rcDesktop.bottom - rcDesktop.top - 200;
		}
	}
	else
	{
		cs.cx = 1000;
		cs.cy = 700;
	}

	cs.x = 0;
	cs.y = 0;

	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	return TRUE;
}

// Do Initial Window Setup
void CMainFrame::InitWindow()
{
	CWnd* pMain = AfxGetMainWnd();
	CTESPCDDoc * pDoc = (CTESPCDDoc *) GetActiveDocument();
	
	OnPanelOff(0); // Turns Off All Panels

	if (pMain != NULL && !pDoc->m_bEnableDialogRelink)	// Verifies Window has been Initialized
	{	
		// Get the main window's menu
		CMenu* pMenu = pMain->GetMenu();

		if (pMenu != NULL)
		{
			#ifdef _DEBUG
			TRACE0("#DEBUG - Removing Disabled Menu Items\n");
			#endif
			pMenu->GetSubMenu(4)->RemoveMenu(10,MF_BYPOSITION);
			pMenu->RemoveMenu(ID_OPERATIONS_REPAIRDIALOG,MF_BYCOMMAND);
			pMenu->RemoveMenu(ID_OPERATIONS_AUTOREPAIRDIALOG,MF_BYCOMMAND);

			pMain->DrawMenuBar();	// force a redraw of the menu bar
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

bool CMainFrame::PeekAndPump()
{
	// Force Translate and Dispatch Messages
	// Read Source on PumpMessage in ThrdCore.Cpp file in MFC\Src 

	static MSG msg;

	while (::PeekMessage(&msg,NULL,0,0,PM_NOREMOVE)) {
		if (!AfxGetApp()->PumpMessage()) {
			::PostQuitMessage(0);
			return FALSE;
		}	
	}

	return TRUE;
}

void CMainFrame::OnSysCommand(UINT nID, LPARAM lParam) 
{
	
	if(nID == SC_CLOSE)
	{
		CTESPCDDoc* pDoc = (CTESPCDDoc *)GetActiveDocument();
		pDoc->SetWorkerThreadForced(TRUE); // Release Thread
		if(pDoc->GetWorkerThread() && pDoc->IsWorkerThreadActive())
		{
			WaitForSingleObject(pDoc->GetWorkerThread(),5000);
		}
	}
	
	CFrameWnd::OnSysCommand(nID, lParam);
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo) 
{
	// Forwards Command Messages to Active View, if it Fails, Sends to All other Views.

	CDocument *pDoc = GetActiveDocument();
    if(pDoc)
    {
		CView * pView = NULL;

//		#ifdef _DEBUG
//		TRACE0("BeginBlock\n");
//		if(GetFocus())
//		{
//			TRACE0(GetFocus()->GetRuntimeClass()->m_lpszClassName);
//			TRACE0("\n");
//		}
//		if(GetActiveWindow())
//		{
//			TRACE0(GetActiveWindow()->GetRuntimeClass()->m_lpszClassName);
//			TRACE0("\n");
//		}
//		if(GetActiveView())
//		{
//			TRACE0(GetActiveView()->GetRuntimeClass()->m_lpszClassName);
//			TRACE0("\n");
//		}
//		#endif

		pView = GetActiveView();
		if(pView)
		{
			if(pView->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
				return TRUE;
		}

		POSITION pos = pDoc->GetFirstViewPosition();

		while(pView = pDoc->GetNextView(pos))
		{	
			if(pView != GetActiveWindow())
			{
				if(pView->OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
					return TRUE;
			}
		}
    }

	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

void CMainFrame::OnAppExit() 
{
	SendMessage(WM_SYSCOMMAND,SC_CLOSE,0);
}

void CMainFrame::OnStepBar(CString *BarText,int MaxCount)
{
	if(MaxCount > 0 && m_wndProgressBar.GetSafeHwnd() == NULL)
	{
		m_wndProgressBar.Create(_T("Progress"), 40, MaxCount, TRUE);
		m_wndProgressBar.SetText(*BarText);
	}
	else if(MaxCount > 0)
	{
		m_wndProgressBar.SetText(*BarText);
		m_wndProgressBar.StepIt();
		PeekAndPump();
	}
	else
	{
		m_wndProgressBar.Clear();
		m_wndProgressBar.DestroyWindow();
	}
}

LRESULT CMainFrame::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{

	if(message == UDM_STEPPBAR)
	{
		OnStepBar((CString*)wParam, (int)lParam);
		return 1;
	}
	else if(message == UDM_PANELON)
	{
		OnPanelOn((int) wParam);
		return 1;
	}
	else if(message == UDM_PANELOFF)
	{
		OnPanelOff((int) wParam);
		return 1;
	}
	else
	{
		return CFrameWnd::WindowProc(message, wParam, lParam);
	}
}


void CMainFrame::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CFrameWnd::OnShowWindow(bShow, nStatus);
	
	if(m_bInitShow && bShow)
	{	
		m_bInitShow = FALSE;
		InitWindow();
	}
	
}


BOOL CMainFrame::ClosePopupWindows()
{
	// IN FUTURE Version use ENUMWINDOWS()

	CWnd *pWnd = GetWindow(GW_HWNDLAST);
	// Close Pop-up Windows
	while (pWnd != NULL)
	{
		if(pWnd->GetOwner()->GetSafeHwnd() == this->GetSafeHwnd() &&
		   pWnd->IsKindOf(RUNTIME_CLASS(CEntryDlg)))
		{
			if(IsWindow(pWnd->GetSafeHwnd()))
				pWnd->PostMessage(UDM_CHECK);
		}

		pWnd = pWnd->GetNextWindow(GW_HWNDPREV);
	}

	return TRUE;
}

BOOL CMainFrame::ClosePopup(const CString strTitle)
{
	// Close Pop-up Window
	CWnd *pWnd = FindWindow(NULL,strTitle);
	if(pWnd)
	{
		if(pWnd->IsKindOf(RUNTIME_CLASS(CEntryDlg)) && 
		   pWnd->GetOwner()->GetSafeHwnd() == this->GetSafeHwnd())
		{
			if(IsWindow(pWnd->GetSafeHwnd()))
				pWnd->PostMessage(UDM_CHECK);
			
			return TRUE;
		}
	}

	return FALSE;
}


BOOL CMainFrame::FocusPopup(const CString strTitle)
{
	// Focus Pop-up Window
	CWnd *pWnd = FindWindow(NULL,strTitle);
	if(pWnd)
	{
		if(pWnd->IsKindOf(RUNTIME_CLASS(CEntryDlg)) && 
		   pWnd->GetOwner()->GetSafeHwnd() == this->GetSafeHwnd())
		{
			pWnd->SetFocus();
			return TRUE;
		}
	}

	return FALSE;
}

/*
	Calling with a parameter as 0 Turns on all Panels 
 */
void CMainFrame::OnPanelOn(int row)
{

	if(row < 0) //Safety Check
		return;
	
	int i = 0;

	switch(row)
	{
	case 0:
		m_bfPanelsShown = PANEL_ALL;

		for(i = 0;i < m_cSplitters;i++)
		{
			if(m_pWndSplitter[i]->IsSideHidden(0))
			{
				m_pWndSplitter[i]->ToggleSide(0);
			}
			if(m_pWndSplitter[i]->IsSideHidden(1))
			{
				m_pWndSplitter[i]->ToggleSide(1);
			}
		}

		m_wndToolBar.GetToolBarCtrl().CheckButton(ID_CNFPANE,TRUE);
		m_wndToolBar.GetToolBarCtrl().CheckButton(ID_WRNPANE,TRUE);
		m_wndToolBar.GetToolBarCtrl().CheckButton(ID_EDTPANE,TRUE);

		break;
	default:
		m_bfPanelsShown |= (PANEL_CONFLICT << (row - 1));
		
		// Check Enable First Splitter
		if((m_bfPanelsShown & PANEL_ALL) && m_pWndSplitter[0]->IsSideHidden(1))
		{
			m_pWndSplitter[0]->ToggleSide(1);
		}

		// Check Enable Second Splitter
		if((m_bfPanelsShown & PANEL_ALL & ~PANEL_CONFLICT) && m_pWndSplitter[1]->IsSideHidden(1))
		{
			m_pWndSplitter[1]->ToggleSide(1);
		}

		if((m_bfPanelsShown & PANEL_CONFLICT) && m_pWndSplitter[1]->IsSideHidden(0))
		{
			m_pWndSplitter[1]->ToggleSide(0);
		}

		if((m_bfPanelsShown & PANEL_WARNING) && m_pWndSplitter[2]->IsSideHidden(0))
		{
			m_pWndSplitter[2]->ToggleSide(0);	
		}

		if((m_bfPanelsShown & PANEL_EDITOR) && m_pWndSplitter[2]->IsSideHidden(1))
		{
			m_pWndSplitter[2]->ToggleSide(1);	
		}

		if(m_bfPanelsShown & PANEL_EDITOR)
		{
			m_wndToolBar.GetToolBarCtrl().CheckButton(ID_EDTPANE,TRUE);
		}

		if(m_bfPanelsShown & PANEL_WARNING)
		{
			m_wndToolBar.GetToolBarCtrl().CheckButton(ID_WRNPANE,TRUE);
		}

		if(m_bfPanelsShown & PANEL_CONFLICT)
		{
			m_wndToolBar.GetToolBarCtrl().CheckButton(ID_CNFPANE,TRUE);
		}

		OnPanelOff(10);

		break;
	}
}

void CMainFrame::OnPanelOff(int row)
{

	if(row < 0) //Safety Check
		return;

	int i = 0;

	switch(row)
	{
	case 0:
		m_bfPanelsShown = PANEL_NONE;

		if(!m_pWndSplitter[0]->IsSideHidden(1))
		{
			m_pWndSplitter[0]->ToggleSide(1);
		}

		for(i = 1;i < m_cSplitters;i++)
		{	// Hide All Except First
			if(!m_pWndSplitter[i]->IsSideHidden(0))
			{
				m_pWndSplitter[i]->ToggleSide(0);
			}
			if(!m_pWndSplitter[i]->IsSideHidden(1))
			{
				m_pWndSplitter[i]->ToggleSide(1);
			}
		}

		m_wndToolBar.GetToolBarCtrl().CheckButton(ID_CNFPANE,FALSE);
		m_wndToolBar.GetToolBarCtrl().CheckButton(ID_WRNPANE,FALSE);
		m_wndToolBar.GetToolBarCtrl().CheckButton(ID_EDTPANE,FALSE);

		break;
	default:
		m_bfPanelsShown &= ~(PANEL_CONFLICT << (row - 1));

		if(!(m_bfPanelsShown & PANEL_ALL) && !m_pWndSplitter[0]->IsSideHidden(1))
		{
			m_pWndSplitter[0]->ToggleSide(1);
		}

		if(!(m_bfPanelsShown & PANEL_ALL & ~PANEL_CONFLICT) && !m_pWndSplitter[1]->IsSideHidden(1))
		{
			m_pWndSplitter[1]->ToggleSide(1);
		}

		if(!(m_bfPanelsShown & PANEL_CONFLICT) && !m_pWndSplitter[1]->IsSideHidden(0))
		{
			m_pWndSplitter[1]->ToggleSide(0);
		}

		if(!(m_bfPanelsShown & PANEL_WARNING) && !m_pWndSplitter[2]->IsSideHidden(0))
		{
			m_pWndSplitter[2]->ToggleSide(0);
		}

		if(!(m_bfPanelsShown & PANEL_EDITOR) && !m_pWndSplitter[2]->IsSideHidden(1))
		{
			m_pWndSplitter[2]->ToggleSide(1);
		}

		if(!(m_bfPanelsShown & PANEL_EDITOR))
		{
			m_wndToolBar.GetToolBarCtrl().CheckButton(ID_EDTPANE,FALSE);
		}

		if(!(m_bfPanelsShown & PANEL_WARNING))
		{
			m_wndToolBar.GetToolBarCtrl().CheckButton(ID_WRNPANE,FALSE);
		}

		if(!(m_bfPanelsShown & PANEL_CONFLICT))
		{
			m_wndToolBar.GetToolBarCtrl().CheckButton(ID_CNFPANE,FALSE);
		}

		break;
	}
}

void CMainFrame::OnCnfpane() 
{
	if(((CTESPCDDoc *)GetActiveDocument())->IsWorkerThreadActive())
		return;

	if(m_bfPanelsShown & PANEL_CONFLICT) // Toggle Conflict Panel
	{
		OnPanelOff(1);
	}
	else
	{
		OnPanelOn(1);
	}
}

void CMainFrame::OnWrnpane() 
{
	if(((CTESPCDDoc *)GetActiveDocument())->IsWorkerThreadActive())
		return;

	if(m_bfPanelsShown & PANEL_WARNING)	// Toggle Warning Panel
	{
		OnPanelOff(2);
	}
	else
	{
		OnPanelOn(2);
	}
}

void CMainFrame::OnEdtpane() 
{
	if(((CTESPCDDoc *)GetActiveDocument())->IsWorkerThreadActive())
		return;

	if(m_bfPanelsShown & PANEL_EDITOR) // Toggle Editor Panel
	{
		OnPanelOff(3);
	}
	else
	{
		OnPanelOn(3);
	}
}