/////////////////////////////////////////////////////////////////////////////
// TESFile.h : header file
//
// Written by Ely Soto <elyvsoto@hotmail.com>
// Copyright (c) 2002. All Rights Reserved.
//
// This code may be used in compiled form in any way you desire. This software
// may not be sold for profit without the authors written consent.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef CTESFILE_H
#define CTESFILE_H

#include "TESEntry.h"

class CTESFile
{
public:
    // ctor(s)
    CTESFile();
	virtual ~CTESFile();

	// members
	
	struct _sExtractOpt
	{
		BOOL bIncCELL;
		BOOL bIncINFO;
		BOOL bIncLAND;
		BOOL bIncPGRD;
	};

	// Opens the Esp File and Prepares for Extraction
	BOOL OpenFile(const CString espfile, const CString espdir);

	// Extracts all the Data from the Esp file and puts it into
	// Public member EspEntryMap
	BOOL ExtractData(BOOL &ForceExit, _sExtractOpt * extractOpt);

	// Parses Data in entry and returns it in two CStringLists
	BOOL ParseEntry(CStringArray *& SubID, CStringArray *& SubData, 
					const CTESEntry EntrytoParse);

	BOOL CTESFile::UpdateEntry(CTESFile::CTESEntry *UpdateEntry, const CStringArray *SubID,
							   const CStringArray *SubData, BOOL SilentMode = false);

	// Deletes Entry within this Esp, Specified by ID, and Name
	BOOL DeleteEntry(const CTESFile::CTESEntry EntrytoDelete, const BOOL SilentMode = false);

	// Removes Entry from Esp Extraction Map
	BOOL RemoveEntry(CTESFile::CTESEntry * EntrytoDelete,const CString ID);

	// Closes the 'FileStream' Object
	// May be Called Once ExtractData has Completed.
	void CloseFile();

	CMapStringToOb m_EntryMap;

protected:
	BOOL GetNextSize(ULONG & cSize);

	// Get Next Sub Entry with ID searchstr
	CString GetNext(const CString strSpecifiedSubID, UCHAR * cRemainingData, CString * strSubIDFound = NULL);

	BOOL CheckDeletions(const CString entryDump); // Checks if this Entry is Marked as Deleted

	// Creates BackupFile for this File
	BOOL CreateBackupFile(CString RealFilePath, BOOL SilentMode); 

	// CString Buffer Helper Function
	BOOL EqualCStringBuffers(CString str1, CString str2);

	//member variables
	BOOL m_bFileopen;
	BOOL m_bBackupCreated;

	CString m_strBackupPath; // Backup Path and Filename

	CString m_strFilename; // Filename for this TESFile
	CString m_strDirectory; // Path for this TESFile
	
	HANDLE m_hFile; // Handle to File Stream

	CPtrArray m_aDeletedEntries;	// Array of Entries that Have been Deleted
	CPtrArray m_aUpdatedEntries;	// Array of Entries that Have been Modified
	
	struct CTESEntryType	// Arrays of Different Types of Entries
	{
		CPtrArray ACTI,ALCH,APPA,ARMO,BODY,BOOK,BSGN,CELL,CLAS,CLOT,CONT,CREA,DIAL,DOOR;
		CPtrArray ENCH,FACT,GMST,INFO,INGR,NPC_,SCPT,WEAP,LAND,LEVC,LEVI,LIGH,LOCK,LTEX;
		CPtrArray MGEF,RACE,GLOB,PGRD,PROB,SOUN,SKIL,SPEL,MISC,STAT,REGN,REPA,SNDG;
	} m_EntryType;
	
};

#endif
