
#include "stdafx.h"
#include "TESPCD.h"
#include "TESOperations.h"
#include "TESFile.h"


CTESOperations::CTESOperations()
{
	m_Descriptions["ACTI"] ="Activator Entry";
	m_Descriptions["ALCH"] ="Alchemy Item Entry";
	m_Descriptions["APPA"] ="Apparatus Entry";
	m_Descriptions["ARMO"] ="Armor Entry";
	m_Descriptions["BODY"] ="Body Entry";
	m_Descriptions["BOOK"] ="Book Entry";
	m_Descriptions["BSGN"] ="BirthSign Entry";
	m_Descriptions["CELL"] ="Cell Entry";
	m_Descriptions["CLAS"] ="Class Settings";
	m_Descriptions["CLOT"] ="Cloth Settings";
	m_Descriptions["CONT"] ="Container Entry";
	m_Descriptions["CREA"] ="Creature Entry";
	m_Descriptions["DIAL"] ="Dialog Topic";
	m_Descriptions["DOOR"] ="Door Entry";
	m_Descriptions["ENCH"] ="Enchantment Settings";
	m_Descriptions["FACT"] ="Faction Settings";
	m_Descriptions["GLOB"] ="Global Settings";
	m_Descriptions["GMST"] ="Gameplay Settings";
	m_Descriptions["INGR"] ="Ingredient Entry";
	m_Descriptions["INFO"] ="Dialog Text";
	m_Descriptions["LAND"] ="Landscape Entry";
	m_Descriptions["LEVC"] ="Creature Leveled List";
	m_Descriptions["LEVI"] ="Item Leveled List";
	m_Descriptions["LIGH"] ="Light Settings";
	m_Descriptions["LOCK"] ="Lockpick Tool";
	m_Descriptions["LTEX"] ="Texture Entry";
	m_Descriptions["MGEF"] ="Magic Effects Entry";
	m_Descriptions["MISC"] ="Misc. Items Entry";
	m_Descriptions["NPC_"] ="NPC Entry";
	m_Descriptions["PGRD"] ="Path Grid";
	m_Descriptions["PROB"] ="Probe Tool";
	m_Descriptions["RACE"] ="Race Settings";
	m_Descriptions["REGN"] ="Region Settings Entry";
	m_Descriptions["REPA"] ="Repair Item Entry";
	m_Descriptions["SCPT"] ="Script";
	m_Descriptions["SKIL"] ="Skill Entry";
	m_Descriptions["SNDG"] ="Sound Gen ";
	m_Descriptions["SOUN"] ="Sound Entry";
	m_Descriptions["SPEL"] ="Spell Entry";
	m_Descriptions["STAT"] ="Static Entry";
	m_Descriptions["WEAP"] ="Weapon Entry";
	
	m_Descriptions["DELE"] ="Marked as DELETED (Removed from Game)";
	m_Descriptions["UNCL"] ="Identical to Default Entry. (Unclean Entry)";
	m_Descriptions["GCHG"] ="Modified from Esm";
	m_Descriptions["POTE"] ="(Potential Conflict)";
	
}

CTESOperations::~CTESOperations()
{
	int cArray, i;
	m_Descriptions.RemoveAll();
	m_aEspArray.RemoveAll(); //These Esps are deleted In the DOC destructor

	cArray = m_aConflictArray.GetSize();
	for(i=0;i < cArray; i++)
		delete (_sConflict*)m_aConflictArray[i];

	m_aConflictArray.RemoveAll();

	cArray = m_aDeletedArray.GetSize();
	for(i=0;i < cArray; i++)
		delete (_sWarning*)m_aDeletedArray[i];

	m_aDeletedArray.RemoveAll();

	cArray = m_aUncleanArray.GetSize();
	for(i=0;i < cArray; i++)
		delete (_sWarning*)m_aUncleanArray[i];

	m_aUncleanArray.RemoveAll();

	cArray = m_aGlobalArray.GetSize();
	for(i=0;i < cArray; i++)
		delete (_sWarning*)m_aGlobalArray[i];

	m_aGlobalArray.RemoveAll();

	cArray = m_aFindArray.GetSize();
	for(i=0;i < cArray; i++)
		delete (_sWarning*)m_aFindArray[i];

	m_aFindArray.RemoveAll();

}

void CTESOperations::AddEsp(CMapStringToOb* Map)
{
	m_aEspArray.Add(Map);
}

void CTESOperations::AddEsm(CMapStringToOb* Map)
{
	m_aEsmArray.Add(Map);
}


CMapStringToOb * CTESOperations::CheckErrors(BOOL &ForceExit,int OpType, 
								_sOptionTypes * ConflictCheckOptions, 
								_sOptionTypes * UncleanConflictCheckOptions, 
								UCHAR CheckFlags /* = CHECK_ALL */)
{
	CMapStringToOb *espMap1, *espMap2;
	int i,j;
	int cEsp = m_aEspArray.GetSize();
	CString strIdentifyOp = "Checking Conflicts ";
	CString barstr = strIdentifyOp + "0% complete";
	
	if(CheckFlags & CHECK_CONFLICTS)
	{

		if(!ForceExit)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cEsp);

		for(i = 0;i < cEsp && !ForceExit;i++)
		{
			espMap1 = (CMapStringToOb*) m_aEspArray.GetAt(i);
			
			for(j = i + 1;j < cEsp && !ForceExit;j++)
			{
				espMap2 = (CMapStringToOb*) m_aEspArray.GetAt(j);
				
				if(ConflictCheckOptions->ACTI == TRUE)
					CheckConflicts(espMap1, espMap2, "ACTI", ForceExit);
				if(ConflictCheckOptions->ALCH == TRUE)
					CheckConflicts(espMap1, espMap2, "ALCH", ForceExit);
				if(ConflictCheckOptions->APPA == TRUE)
					CheckConflicts(espMap1, espMap2, "APPA", ForceExit);
				if(ConflictCheckOptions->ARMO == TRUE)
					CheckConflicts(espMap1, espMap2, "ARMO", ForceExit);
				if(ConflictCheckOptions->BODY == TRUE)
					CheckConflicts(espMap1, espMap2, "BODY", ForceExit);
				if(ConflictCheckOptions->BOOK == TRUE)
					CheckConflicts(espMap1, espMap2, "BOOK", ForceExit);
				if(ConflictCheckOptions->BSGN == TRUE)
					CheckConflicts(espMap1, espMap2, "BSGN", ForceExit);
				if(ConflictCheckOptions->CLAS == TRUE)
					CheckConflicts(espMap1, espMap2, "CLAS", ForceExit);
				if(ConflictCheckOptions->CLOT == TRUE)
					CheckConflicts(espMap1, espMap2, "CLOT", ForceExit);
				if(ConflictCheckOptions->CONT == TRUE)
					CheckConflicts(espMap1, espMap2, "CONT", ForceExit);
				if(ConflictCheckOptions->CREA == TRUE)
					CheckConflicts(espMap1, espMap2, "CREA", ForceExit);
				if(ConflictCheckOptions->DOOR == TRUE)
					CheckConflicts(espMap1, espMap2, "DOOR", ForceExit);
				if(ConflictCheckOptions->ENCH == TRUE)
					CheckConflicts(espMap1, espMap2, "ENCH", ForceExit);
				if(ConflictCheckOptions->FACT == TRUE)
					CheckConflicts(espMap1, espMap2, "FACT", ForceExit);
				if(ConflictCheckOptions->GLOB == TRUE)
					CheckConflicts(espMap1, espMap2, "GLOB", ForceExit);
				if(ConflictCheckOptions->GMST == TRUE)
					CheckConflicts(espMap1, espMap2, "GMST", ForceExit);
				if(ConflictCheckOptions->INFO == TRUE)
					CheckConflicts(espMap1, espMap2, "INFO", ForceExit);
				if(ConflictCheckOptions->INGR == TRUE)
					CheckConflicts(espMap1, espMap2, "INGR", ForceExit);
				if(ConflictCheckOptions->LAND == TRUE)
					CheckConflicts(espMap1, espMap2, "LAND", ForceExit);
				if(ConflictCheckOptions->LEVC == TRUE)
					CheckConflicts(espMap1, espMap2, "LEVC", ForceExit);
				if(ConflictCheckOptions->LEVI == TRUE)
					CheckConflicts(espMap1, espMap2, "LEVI", ForceExit);
				if(ConflictCheckOptions->LIGH == TRUE)
					CheckConflicts(espMap1, espMap2, "LIGH", ForceExit);
				if(ConflictCheckOptions->LOCK == TRUE)
					CheckConflicts(espMap1, espMap2, "LOCK", ForceExit);
				if(ConflictCheckOptions->MGEF == TRUE)
					CheckConflicts(espMap1, espMap2, "MGEF", ForceExit);
				if(ConflictCheckOptions->MISC == TRUE)
					CheckConflicts(espMap1, espMap2, "MISC", ForceExit);
				if(ConflictCheckOptions->NPC_ == TRUE)
					CheckConflicts(espMap1, espMap2, "NPC_", ForceExit);
				if(ConflictCheckOptions->PGRD == TRUE)
					CheckConflicts(espMap1, espMap2, "PGRD", ForceExit);
				if(ConflictCheckOptions->PROB == TRUE)
					CheckConflicts(espMap1, espMap2, "PROB", ForceExit);
				if(ConflictCheckOptions->RACE == TRUE)
					CheckConflicts(espMap1, espMap2, "RACE", ForceExit);
				if(ConflictCheckOptions->REGN == TRUE)
					CheckConflicts(espMap1, espMap2, "REGN", ForceExit);
				if(ConflictCheckOptions->REPA == TRUE)
					CheckConflicts(espMap1, espMap2, "REPA", ForceExit);
				if(ConflictCheckOptions->SCPT == TRUE)
					CheckConflicts(espMap1, espMap2, "SCPT", ForceExit);
				if(ConflictCheckOptions->SKIL == TRUE)
					CheckConflicts(espMap1, espMap2, "SKIL", ForceExit);
				if(ConflictCheckOptions->SNDG == TRUE)
					CheckConflicts(espMap1, espMap2, "SNDG", ForceExit);
				if(ConflictCheckOptions->SOUN == TRUE)
					CheckConflicts(espMap1, espMap2, "SOUN", ForceExit);
				if(ConflictCheckOptions->SPEL == TRUE)
					CheckConflicts(espMap1, espMap2, "SPEL", ForceExit);
				if(ConflictCheckOptions->STAT == TRUE)
					CheckConflicts(espMap1, espMap2, "STAT", ForceExit);
				if(ConflictCheckOptions->WEAP == TRUE)
					CheckConflicts(espMap1, espMap2, "WEAP", ForceExit);
				
			}
			
			barstr.Format(strIdentifyOp + "%d%% complete", i*100/cEsp);
			if(!ForceExit)
				SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cEsp);

			//If SingleVersus Type Check, break - Master Plugin must be added first
			if(OpType == 1)
				break;
		}

		if(!ForceExit)
		{
			//Destroy the Bar
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,0);
		}
	}
	//---------------------------------------------------------------------------------

	if(CheckFlags & ~CHECK_CONFLICTS)
	{

		//If SingleVersus Type
		if(OpType == 1)
			cEsp = 1;
		else
			cEsp = m_aEspArray.GetSize();

		if(CheckFlags & CHECK_GLOBAL)
			strIdentifyOp = "Checking Global Changes ";
		else
			strIdentifyOp = "Checking Warnings ";

			
		barstr = strIdentifyOp + "0% complete";
		
		if(!ForceExit)
		{
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cEsp);
		}

		for(i = 0;i < cEsp && !ForceExit;i++)
		{
			espMap1 = (CMapStringToOb*) m_aEspArray.GetAt(i);

			CheckWarnings(ForceExit,espMap1,UncleanConflictCheckOptions, CheckFlags);

			barstr.Format(strIdentifyOp + "%d%% complete", i*100/cEsp);
			if(!ForceExit)
				SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cEsp);

		}

		if(!ForceExit)
		{
			//Destroy the Bar
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,0);
		}
	}

	m_RetData["CONF"] = (CObject*)&m_aConflictArray;
	m_RetData["DELE"] = (CObject*)&m_aDeletedArray;
	m_RetData["UNCL"] = (CObject*)&m_aUncleanArray;
	m_RetData["GCHG"] = (CObject*)&m_aGlobalArray;


	return &m_RetData;
}

void CTESOperations::CheckConflicts(CMapStringToOb* espMap1, 
								   CMapStringToOb* espMap2, 
								   CString IDtype,
								   BOOL &ForceExit)
{
	CMapStringToOb *tempMap;
	CPtrArray *pEsmTypeArray;
	int cEsmTypeArray, cEsmArray;
	BOOL bConfFound = FALSE;
	CString *pEsmFilename, *pEsmPath;

	int i, j, x, y;
	int cList1, cList2;
	CString *filename1, *filename2, *path1, *path2;
	CString strTemp1;
	
	WIN32_FILE_ATTRIBUTE_DATA fileattrib1, fileattrib2;
	_GET_FILEEX_INFO_LEVELS filelevels = GetFileExInfoStandard;//GetFileExMaxInfoLevel;

	CTESEntry *pEntry1, *pEntry2, *pEsmEntry;
	_sConflict *pConflict;

	CPtrArray* pPtrArray1 = ((CPtrArray*)(*espMap1)[IDtype]);
	CPtrArray* pPtrArray2 = ((CPtrArray*)(*espMap2)[IDtype]);
	
	cList1 = pPtrArray1->GetSize();
	cList2 = pPtrArray2->GetSize();

	filename1 = ((CString*)(*espMap1)["FILE"]);
	path1 = ((CString*)(*espMap1)["PATH"]);
	filename2 = ((CString*)(*espMap2)["FILE"]);
	path2 = ((CString*)(*espMap2)["PATH"]);

	//-----------------------------------------------------------------
	//Check for Esp Name-Data Conflicts and add to m_aConfArray
	for(i = 0;i < cList1 && !ForceExit;i++)
	{
		pEntry1 = (CTESEntry*) pPtrArray1->GetAt(i);

		for(j = 0;j < cList2 && !ForceExit;j++)
		{
			pEntry2 = (CTESEntry*) pPtrArray2->GetAt(j);

			bConfFound = FALSE;

			//Check for Esp to Esp Conflicts
			if(pEntry1->EntryName == pEntry2->EntryName )
			{
				if(IDtype == "LAND" ||
				   !EqualCStringBuffers(pEntry1->EntryData ,pEntry2->EntryData))
				{
					if(pEntry1->pstrExtra != NULL && pEntry2->pstrExtra != NULL)
					{	// These Variables Function as an alternate Compare for Conflict Checks
						bConfFound = FALSE;
						if(*pEntry1->pstrExtra == *pEntry2->pstrExtra)
						{
							bConfFound = TRUE;
						}
					}
					else
					{
						bConfFound = TRUE;
					}
					
					if(IDtype == "INFO")
					{	// Reported Conflicts Assume they Will use Dialog Merger.
						// Reports Conflicts with Entries that are Not Identical to
						// Entries in Master Files
						if(GetEntry(GetEsmOrigin(ForceExit,pEntry1->EntryName,IDtype),
									pEntry1->EntryName,IDtype,pEsmEntry,FALSE))
						{
							if(!EqualCStringBuffers(pEsmEntry->EntryData, pEntry1->EntryData) &&
							   !EqualCStringBuffers(pEsmEntry->EntryData, pEntry2->EntryData))
							{
								bConfFound = TRUE;
							}
							else
							{
								bConfFound = FALSE;
							}
						}
					}
				}
			}

			if(bConfFound)
			{
				// **Conflict Found**
				pConflict = new _sConflict;
				pConflict->ID = IDtype;
				pConflict->Description = m_Descriptions[IDtype];
				pConflict->Name = pEntry1->EntryName;

				if(IDtype == "LAND")
					pConflict->Description += " " + m_Descriptions["POTE"];
				
				GetFileAttributesEx(*path1 + *filename1,filelevels,&fileattrib1);
				GetFileAttributesEx(*path2 + *filename2,filelevels,&fileattrib2);
		 
				if(CTime(fileattrib1.ftLastWriteTime) > CTime(fileattrib2.ftLastWriteTime))
				{
					pConflict->Filename1 = *filename1;
					pConflict->Filename2 = *filename2;
				}
				else
				{
					pConflict->Filename1 = *filename2;
					pConflict->Filename2 = *filename1;
				}
				
				pConflict->Filename3 = "None";

				// ---- Search for Original Esm ----

				cEsmArray = m_aEsmArray.GetSize();
				fileattrib2.ftLastWriteTime.dwHighDateTime = NULL;
				fileattrib2.ftLastWriteTime.dwLowDateTime = NULL;

				for(x = 0;x < cEsmArray;x++)
				{

					tempMap = (CMapStringToOb*) m_aEsmArray.GetAt(x);

					pEsmFilename = ((CString*)(*tempMap)["FILE"]);
					pEsmPath = ((CString*)(*tempMap)["PATH"]);
					
					GetFileAttributesEx(*pEsmPath + *pEsmFilename,filelevels,&fileattrib1);

					tempMap->Lookup(IDtype,(CObject*&)pEsmTypeArray);
					
					cEsmTypeArray = pEsmTypeArray->GetSize();
					for(y = 0;y < cEsmTypeArray; y++)
					{
						if(((CTESEntry*)pEsmTypeArray->GetAt(y))->EntryName == pEntry1->EntryName)
						{
							if(CTime(fileattrib1.ftLastWriteTime) > CTime(fileattrib2.ftLastWriteTime))
							{
								fileattrib2 = fileattrib1;
								pConflict->Filename3 = *pEsmFilename;
							}
						}
					}
				}

				m_aConflictArray.Add(pConflict);
			}

		}
	}
}

BOOL CTESOperations::CheckWarnings(BOOL &ForceExit, 
								   CMapStringToOb * tempEspMap,
								   _sOptionTypes * UncleanCheckOptions, 
								   UCHAR CheckFlags /* = CHECK_ALL */)
{
	POSITION posMap;
	CPtrArray * pEspPtrArray;
	CPtrArray * pEsmTypeArray;
	CString mapKey;
	CString * pEspFilename, * pEsmFilename, * pEsmPath;
	CString strCurEsm, strDefEsm;
	int cEspPtrArray, cEsmTypeArray, cEsmArray;
	int x,y,j;
	BOOL bNewUncleanWarning, bNewGlobalChange;
	CMapStringToOb *tempEsmMap;
	CTESEntry * pEsmEntry;
	CTESEntry * pEntry;
	_sWarning * pWarning1, * pWarning2;
	
	WIN32_FILE_ATTRIBUTE_DATA fileattrib1, fileattrib2;
	_GET_FILEEX_INFO_LEVELS filelevels = GetFileExInfoStandard;//GetFileExMaxInfoLevel;

	posMap = tempEspMap->GetStartPosition();
	pEspFilename = ((CString*)(*tempEspMap)["FILE"]);

	while(posMap != NULL && !ForceExit)
	{
		tempEspMap->GetNextAssoc(posMap,mapKey,(CObject*&)pEspPtrArray);

		if(mapKey != "FILE" && mapKey != "PATH")
		{
			cEspPtrArray = pEspPtrArray->GetSize();

			for(j = 0;j < cEspPtrArray && !ForceExit;j++)
			{
				pEntry = (CTESEntry*) pEspPtrArray->GetAt(j);

				//----Detection of Unclean Entries.----

				bNewUncleanWarning = true;
				bNewGlobalChange = true;
				strCurEsm = "None";
				strDefEsm = "None";
				cEsmArray = m_aEsmArray.GetSize();
				fileattrib2.ftLastWriteTime.dwHighDateTime = NULL;
				fileattrib2.ftLastWriteTime.dwLowDateTime = NULL;

				for(x = 0;x < cEsmArray && !ForceExit;x++)
				{
					tempEsmMap = (CMapStringToOb*) m_aEsmArray.GetAt(x);

					pEsmFilename = ((CString *) (*tempEsmMap)["FILE"]);
					pEsmPath = ((CString *) (*tempEsmMap)["PATH"]);
					
					GetFileAttributesEx(*pEsmPath + *pEsmFilename,filelevels,&fileattrib1);

					tempEsmMap->Lookup(mapKey,(CObject*&)pEsmTypeArray);
					
					if(	mapKey == "DIAL" || mapKey == "LAND" ||
						mapKey == "LTEX" || mapKey == "CELL")
					{
						continue;
					}

					if(mapKey == "INFO" && !UncleanCheckOptions->INFO)
					{
						continue;
					}

					cEsmTypeArray = pEsmTypeArray->GetSize();
					for(y = 0;y < cEsmTypeArray && !ForceExit; y++)
					{
						pEsmEntry = ((CTESEntry *) pEsmTypeArray->GetAt(y));
						if(pEsmEntry->EntryName == pEntry->EntryName)
						{	
							if(CTime(fileattrib1.ftLastWriteTime) > CTime(fileattrib2.ftLastWriteTime))
							{
								strDefEsm = *pEsmFilename;
								
								if((CheckFlags & CHECK_UNCLEAN) && pEntry->EntryData != "" &&
									EqualCStringBuffers(pEsmEntry->EntryData, pEntry->EntryData))
								{
								
									// ***Unclean Entry Found***
									if(bNewUncleanWarning)
									{
										pWarning1 = new _sWarning;
										bNewUncleanWarning = false;
										
										pWarning1->ID = mapKey;
										pWarning1->Description = m_Descriptions[mapKey] + " " + m_Descriptions["UNCL"];
										pWarning1->Name = pEntry->EntryName;
										pWarning1->Filename = *pEspFilename;

									}
									fileattrib2 = fileattrib1;
									
									pWarning1->OrigFile = *pEsmFilename;

									strCurEsm = *pEsmFilename;
								
								}
								
								if((CheckFlags & CHECK_GLOBAL) && pEntry->EntryData != "" &&
									!EqualCStringBuffers(pEsmEntry->EntryData, pEntry->EntryData))
								{
									// ***Found Global Change***								

									if(bNewGlobalChange)
									{
										pWarning2 = new _sWarning;
										bNewGlobalChange = false;
										
										pWarning2->ID = mapKey;
										pWarning2->Description = m_Descriptions[mapKey] + " " + m_Descriptions["GCHG"];
										pWarning2->Name = pEntry->EntryName;
										pWarning2->Filename = *pEspFilename;
									}
									fileattrib2 = fileattrib1;

									pWarning2->OrigFile = *pEsmFilename;

									strCurEsm = *pEsmFilename;
								}							
							}
							break;
						}
					}
				}

				if(!bNewUncleanWarning)
				{
					m_aUncleanArray.Add(pWarning1);
				}
				if(!bNewGlobalChange)
				{
					m_aGlobalArray.Add(pWarning2);
				}
				
				if((CheckFlags & CHECK_DELETIONS) && pEntry->bIsDeleted)
				{
					// *** Report Deleted Item ***
					pWarning1 = new _sWarning;
					pWarning1->ID = mapKey;
					pWarning1->Description = m_Descriptions[mapKey] + " " + m_Descriptions["DELE"];
					pWarning1->Name = pEntry->EntryName;
					pWarning1->Filename = *pEspFilename;
					if(strCurEsm != "None")
						pWarning1->OrigFile = strCurEsm;
					else
						pWarning1->OrigFile = strDefEsm;
					m_aDeletedArray.Add(pWarning1);

				}
			}
		}
	}

	return true;
}

CPtrArray * CTESOperations::FindObjects(BOOL &ForceExit, 
									   CString strObjName, 
									   BOOL bMatchcase,
									   BOOL bMatchword,
									   BOOL bLookdata,
									   BOOL bEsmSearch)
{
	
	CMapStringToOb *tempMap;
	CPtrArray * pPArray, * pTESFileArray;
	CString strCmpData, strKey, tempName, * tempData, * tempID, * pFilename, strOrgEsm;
	int cPArray, cMapTypes;
	POSITION posMap;
	BOOL bFoundMatch = false;
	_sEditRow * pEditRow;
	CTESEntry * pFindEntry;

	CString barstr = "Searching... 0% complete";

	if(bEsmSearch)
	{
		pTESFileArray = &m_aEsmArray;
	}
	else
	{
		pTESFileArray = &m_aEspArray;
	}

	int cEsp = pTESFileArray->GetSize();

	if(!ForceExit)
	{
		SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cEsp);
	}

	for(int i = 0;i < cEsp && !ForceExit;i++)
	{ // For Each

		tempMap = (CMapStringToOb*) pTESFileArray->GetAt(i);

		pFilename = ((CString*)(*tempMap)["FILE"]);
	
		posMap = tempMap->GetStartPosition();
		cMapTypes = tempMap->GetCount();

		for(int k = 0;k < cMapTypes && !ForceExit;k++)
		{ // For Each List Type
			tempMap->GetNextAssoc(posMap,strKey,(CObject*&)pPArray);
			if(strKey != "FILE" && strKey != "PATH")
			{
				cPArray = pPArray->GetSize();

				for(int j = 0;j < cPArray && !ForceExit;j++)
				{	// For Each Entry within List Type
					pFindEntry =  (CTESEntry *) pPArray->GetAt(j);
					
					tempName = pFindEntry->EntryName;
					tempData = &pFindEntry->EntryData;
					tempID = &strKey;
					
					if(!bMatchcase)
					{
						tempName.MakeUpper();
						strObjName.MakeUpper();
					}
					
					if(!bMatchword)
					{
						int cObjEntry = strObjName.GetLength();
						int cTempEntry = tempName.GetLength();
						for(int n = 0;n < cTempEntry && cTempEntry - n > cObjEntry && !ForceExit;n++) 
						{
							strCmpData = tempName.Mid(n,cObjEntry);
							
							if(!bMatchcase)
								strCmpData.MakeUpper();

							if(strCmpData == strObjName)
							{
								// Found Match
								bFoundMatch = true;				
								break;
							}
						}
					}
					else if((bMatchword && (tempName == strObjName)))
					{
						// Found Match
						bFoundMatch = true;
					}

					if(bLookdata)
					{
						int cObjEntry = strObjName.GetLength();
						int cTempEntry = tempData->GetLength();
						for(int n = 0;n < cTempEntry && cTempEntry - n >= cObjEntry && !ForceExit;n++) 
						{
							strCmpData = tempData->Mid(n,cObjEntry);
							
							if(!bMatchcase)
								strCmpData.MakeUpper();

							if(strCmpData == strObjName)
							{
								// Found Match
								bFoundMatch = true;				
								break;
							}
						}

						if(!bFoundMatch)
						{
							cTempEntry = tempID->GetLength();
							for(int n = 0;n < cTempEntry && cTempEntry - n >= cObjEntry && !ForceExit;n++) 
							{
								strCmpData = tempID->Mid(n,cObjEntry);
								
								if(!bMatchcase)
									strCmpData.MakeUpper();

								if(strCmpData == strObjName)
								{
									// Found Match
									bFoundMatch = true;				
									break;
								}
							}
						}

					}

					if(bFoundMatch)
					{
						pEditRow = new _sEditRow;

						pEditRow->ID = strKey;
						pEditRow->Name = pFindEntry->EntryName;
						pEditRow->Description = m_Descriptions[strKey];
						
						strOrgEsm = "";

						if(!bEsmSearch)
						{
							strOrgEsm = GetEsmOrigin(ForceExit, pFindEntry->EntryName, strKey);
							pEditRow->Filename = *pFilename;
						}
						else
						{
							strOrgEsm = *pFilename;
							pEditRow->Filename = "None";
						}
						
						if(strOrgEsm == "")
						{
							strOrgEsm = "None";
						}

						pEditRow->OrigFile = strOrgEsm;

						m_aFindArray.Add(pEditRow);
						
						bFoundMatch = false;
					}

				}
			}
		}


		barstr.Format("Searching... %d%% complete", i*100/cEsp);
		if(!ForceExit)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cEsp);
	}

	if(!ForceExit)
	{
		//Destroy the Bar
		SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,0);
	}

	return &m_aFindArray;
}


BOOL CTESOperations::EqualCStringBuffers(CString str1, CString str2)
{	// Returns 0 if Equal
	int str1len = str1.GetLength();
	int str2len = str2.GetLength();

	if(str1len != str2len)
		return false;

	for(int i = 0;i < str1len;i++)
	{
		if(str1[i] != str2[i])
			return false;
	}

	return true;
}

CString CTESOperations::GetDesc(CString mapKey)
{
	return m_Descriptions[mapKey];
}

CString CTESOperations::GetEsmOrigin(BOOL &ForceExit, CString strEntryName, CString mapKey)
{

	int cEsmTypeArray, cEsmArray;
	CPtrArray *pEsmTypeArray;
	CString *pEsmFilename, *pEsmPath;
	CString strCurEsm = "";
	int x,y;
	CMapStringToOb *tempEsmMap;
	CTESEntry * pEsmEntry;
	
	WIN32_FILE_ATTRIBUTE_DATA fileattrib1, fileattrib2;
	_GET_FILEEX_INFO_LEVELS filelevels = GetFileExInfoStandard;//GetFileExMaxInfoLevel;

	cEsmArray = m_aEsmArray.GetSize();
	fileattrib2.ftLastWriteTime.dwHighDateTime = NULL;
	fileattrib2.ftLastWriteTime.dwLowDateTime = NULL;

	for(x = 0;x < cEsmArray && !ForceExit;x++)
	{
		tempEsmMap = (CMapStringToOb*) m_aEsmArray.GetAt(x);

		pEsmFilename = ((CString *) (*tempEsmMap)["FILE"]);
		pEsmPath = ((CString *) (*tempEsmMap)["PATH"]);
		
		GetFileAttributesEx(*pEsmPath + *pEsmFilename,filelevels,&fileattrib1);

		tempEsmMap->Lookup(mapKey,(CObject*&)pEsmTypeArray);
		
		cEsmTypeArray = pEsmTypeArray->GetSize();
		for(y = 0;y < cEsmTypeArray && !ForceExit; y++)
		{
			pEsmEntry = ((CTESEntry *) pEsmTypeArray->GetAt(y));
			if(pEsmEntry->EntryName == strEntryName)
			{	
				if(CTime(fileattrib1.ftLastWriteTime) > CTime(fileattrib2.ftLastWriteTime))
				{
					fileattrib2 = fileattrib1;
					strCurEsm = *pEsmFilename;
				}
				break;
			}
		}

	}

	if(strCurEsm == "")
		return "None";
	else
		return strCurEsm;
}

BOOL CTESOperations::GetEntry(CString strFilename, CString strEntryName,
							 CString strEntryType, CTESEntry *&pEntry,
							 BOOL bFromEsp)
{
	CMapStringToOb *tempMap;
	CPtrArray *pPtrList;
	CString strName, * pEsmFilename;
	int cPtrList;
	CPtrArray *pDataArray;

	if(bFromEsp)
	{
		pDataArray = &m_aEspArray;
	}
	else
	{
		pDataArray = &m_aEsmArray;
	}

	int arraysize = pDataArray->GetSize();

	for(int i = 0;i < arraysize;i++)
	{
		tempMap = (CMapStringToOb*) pDataArray->GetAt(i);
		
		pEsmFilename = ((CString *) (*tempMap)["FILE"]);;
		if(*pEsmFilename == strFilename)
		{
			tempMap->Lookup(strEntryType,(CObject*&)pPtrList);
			
			cPtrList = pPtrList->GetSize();
			for(int j = 0;j < cPtrList; j++)
			{
				strName = ((CTESEntry*)pPtrList->GetAt(j))->EntryName;

				if(strName == strEntryName)
				{
					pEntry = (CTESEntry*)pPtrList->GetAt(j);
					return true;
				}
			}
		}
	}
	
	return false;
}
