/////////////////////////////////////////////////////////////////////////////
// CTESOperations.h : header file
//
// Written by Ely Soto <elyvsoto@hotmail.com>
// Copyright (c) 2002. All Rights Reserved.
//
// This code may be used in compiled form in any way you desire. This software
// may not be sold for profit without the authors written consent.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef CTESOperations_H
#define CTESOperations_H

#define CHECK_CONFLICTS 0x01
#define CHECK_DELETIONS 0x02
#define CHECK_UNCLEAN	0x04
#define CHECK_GLOBAL	0x08
#define CHECK_ALL		(CHECK_UNCLEAN | CHECK_DELETIONS | CHECK_CONFLICTS)

#include "TESEntry.h"

class CTESOperations
{
public:
	CTESOperations();
	virtual ~CTESOperations();

	// member functions
	void AddEsp(CMapStringToOb*);
	void AddEsm(CMapStringToOb*);
	
	// Conflict Data returned for Display on Grid
	struct _sConflict
	{
		CString ID;
		CString Description;
		CString Name;
		CString Filename1;
		CString Filename2;
		CString Filename3;
	};

	// Warning Data returned for Display on Grid
	struct _sWarning
	{
		CString ID;
		CString Description;
		CString Name;
		CString Filename;
		CString OrigFile;
	};

	// Editor Data returned for Display on Grid
	typedef _sWarning _sEditRow;

	// Used to Specify which Conflicts will be Checked
	struct _sOptionTypes
	{
		BOOL ACTI;
		BOOL ALCH;
		BOOL APPA;
		BOOL ARMO;
		BOOL BODY;
		BOOL BOOK;
		BOOL BSGN;
		BOOL CLAS;
		BOOL CLOT;
		BOOL CONT;
		BOOL CREA;
		BOOL DOOR;
		BOOL ENCH;
		BOOL FACT;
		BOOL GLOB;
		BOOL GMST;
		BOOL INFO;
		BOOL INGR;
		BOOL LAND;
		BOOL LEVC;
		BOOL LEVI;
		BOOL LIGH;
		BOOL LOCK;
		BOOL MGEF;
		BOOL MISC;
		BOOL NPC_;
		BOOL PGRD;
		BOOL PROB;
		BOOL RACE;
		BOOL REGN;
		BOOL REPA;
		BOOL SCPT;
		BOOL SKIL;
		BOOL SNDG;
		BOOL SOUN;
		BOOL SPEL;
		BOOL STAT;
		BOOL WEAP;
	};

	
	// Main Error Check - Calls Conflict and Warning Check
	CMapStringToOb * CheckErrors(BOOL &ForceExit,int OpType, 
									_sOptionTypes * ConflictCheckOptions, 
									_sOptionTypes * UncleanCheckOptions, 
									UCHAR CheckFlags = CHECK_ALL);

	// Returns Description of this Type of Entry
	CString GetDesc(CString mapKey);

	// Locates Latest Esm Filename with Matching Entry
	CString GetEsmOrigin(BOOL &ForceExit, CString strEntryName, CString mapKey);

	// Searches and Locates Data
	CPtrArray * FindObjects(BOOL &ForceExit, CString strObjName, 
							BOOL bMatchcase = FALSE, BOOL bMatchword = TRUE,
							BOOL bLookdata = FALSE, BOOL bEsmSearch = FALSE);

protected:
	
	// Searches and Reports Conflicts
	void CheckConflicts(CMapStringToOb* espMap1, CMapStringToOb* espMap2,
							  CString IDtype,BOOL &ForceExit);

	// Searches and Reports Warnings
	BOOL CheckWarnings(BOOL &ForceExit, CMapStringToOb * tempEspMap, _sOptionTypes * UncleanCheckOptions, UCHAR CheckFlags = CHECK_ALL);

	// CString Buffer Helper Function
	BOOL EqualCStringBuffers(CString str1, CString str2);


	BOOL GetEntry(CString strFilename, CString strEntryName,
							 CString strEntryType, CTESEntry *&pEntry,
							 BOOL bFromEsp);

	// member variables
	
	CMapStringToString m_Descriptions;	// HashMap to contain Descriptions of Entries
	
	// Arrays of CTESFile Objects
	CPtrArray m_aEspArray;
	CPtrArray m_aEsmArray;
	
	// Arrays to Contain Conflict, Warning, Etc. Structures
	CPtrArray m_aConflictArray;
	CPtrArray m_aDeletedArray;
	CPtrArray m_aUncleanArray;
	CPtrArray m_aGlobalArray;
	CPtrArray m_aFindArray;

	// HashMap to Contain Return Arrays
	CMapStringToOb m_RetData;
};

#endif
