// TESPCDDoc.cpp : implementation of the CTESPCDDoc class
//

#include "stdafx.h"
#include "TESPCD.h"

#include "ViewFileDlg.h"
#include "RepairToolDlg.h"
#include "TESDialogOp.h"
#include "TESOperations.h"
#include "TESFile.h"
#include "PPageID.h"
#include "PPageOP.h"
#include "FindObject.h"

#include "TESPCDDoc.h"
#include "ConflictView.h"
#include "WarningView.h"
#include "EditorView.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTESPCDDoc

IMPLEMENT_DYNCREATE(CTESPCDDoc, CDocument)

BEGIN_MESSAGE_MAP(CTESPCDDoc, CDocument)
	//{{AFX_MSG_MAP(CTESPCDDoc)
	ON_COMMAND(ID_FILE_OPEN, OnFileChooseplugins)
	ON_COMMAND(ID_FILE_CHOOSEMASTER, OnFileChoosemaster)
	ON_UPDATE_COMMAND_UI(ID_FILE_CHOOSEMASTER, OnUpdateFileChoosemaster)
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_COMMAND(ID_OPERATIONS_EXTRACTDATA, OnOperationsExtractdata)
	ON_COMMAND(ID_OPERATIONS_CHECKCONFLICTS, OnOperationsCheckconflicts)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_COMMAND(ID_CONTEXTADD, OnContextadd)
	ON_COMMAND(ID_CONTEXTREMOVE, OnContextremove)
	ON_COMMAND(ID_OPERATIONS_OPTIONS, OnOperationsOptions)
	ON_COMMAND(ID_FILE_CHOOSEACTIVEPLUGINS, OnFileChooseactiveplugins)
	ON_COMMAND(ID_OPERATIONS_AUTOCLEAN, OnOperationsAutoclean)
	ON_COMMAND(ID_OPERATIONS_CHECKALL, OnOperationsCheckall)
	ON_COMMAND(ID_OPERATIONS_CHECKDELETED, OnOperationsCheckdeleted)
	ON_COMMAND(ID_OPERATIONS_CHECKUNCLEAN, OnOperationsCheckunclean)
	ON_COMMAND(ID_OPERATIONS_CHECKALLGLOBAL, OnOperationsCheckAllGlobalChanges)
	ON_COMMAND(ID_OPERATIONS_FINDOBJECT, OnOperationsFindobject)
	ON_COMMAND(ID_OPERATIONS_REPAIRDIALOG, OnOperationsRepairdialog)
	ON_COMMAND(ID_OPERATIONS_AUTOREPAIRDIALOG, OnOperationsAutorepairdialog)
	ON_COMMAND(ID_OPERATIONS_VIEWALLESPDATA, OnOperationsViewallespdata)
	ON_COMMAND(ID_OPERATIONS_VIEWALLESMDATA, OnOperationsViewallesmdata)
	ON_COMMAND(ID_OPERATIONS_VIEWFILEDATA, OnOperationsViewfiledata)
	ON_COMMAND(ID_OPERATIONS_PREPARECHECK, PrepareCheck)
	ON_COMMAND(ID_OPERATIONS_CLEARESMDATA, OnOperationsClearesmdata)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTESPCDDoc construction/destruction

CTESPCDDoc::CTESPCDDoc()
{
	m_nOperationType = FULLCHECK;
	m_SingleMaster = NULL;
	m_FilesRoot = NULL;
	m_bWorkerThreadForceExit = FALSE;
	m_bWorkerThreadActive = FALSE;
	m_bEsmExtracted = FALSE;
	m_CurOpFlags = NULL;
	m_bRepairAuto = FALSE;
	m_strDlgEspFile = "";
	m_strDlgEspPath = "";
	m_bEnableDialogRelink = FALSE;

	m_cFindHistory = sizeof(m_strFind) / sizeof(m_strFind[0]);
	for(int i = 0;i < m_cFindHistory;i++)
	{
		m_strFind[i] = "";
	}

	TCHAR retPath[MAX_PATH];
	
	if (((CTESPCDApp*)AfxGetApp())->GetRegKey(HKEY_LOCAL_MACHINE, 
		_T("SOFTWARE\\Bethesda Softworks\\Morrowind"),
		_T("Installed Path"), retPath) == ERROR_SUCCESS)
	{
		m_strDataFilesPath = retPath;
		m_strDataFilesPath += _T("\\Data Files\\\0");
		m_strBasePath = retPath;

	}
	else
	{
		m_strDataFilesPath = TESDEFPATH;
		m_strBasePath = "";
	}

	// DEFAULT VALUES
	m_ConflictCheckTypes.ACTI = TRUE;
	m_ConflictCheckTypes.ALCH = TRUE;
	m_ConflictCheckTypes.APPA = TRUE;
	m_ConflictCheckTypes.ARMO = TRUE;
	m_ConflictCheckTypes.BODY = TRUE;
	m_ConflictCheckTypes.BOOK = TRUE;
	m_ConflictCheckTypes.BSGN = TRUE;
	m_ConflictCheckTypes.CLAS = TRUE;
	m_ConflictCheckTypes.CLOT = TRUE;
	m_ConflictCheckTypes.CONT = TRUE;
	m_ConflictCheckTypes.CREA = TRUE;
	m_ConflictCheckTypes.DOOR = TRUE;
	m_ConflictCheckTypes.ENCH = TRUE;
	m_ConflictCheckTypes.FACT = TRUE;
	m_ConflictCheckTypes.GLOB = TRUE;
	m_ConflictCheckTypes.GMST = TRUE;
	m_ConflictCheckTypes.INFO = TRUE;
	m_ConflictCheckTypes.INGR = TRUE;
	m_ConflictCheckTypes.LAND = TRUE;
	m_ConflictCheckTypes.LEVC = TRUE;
	m_ConflictCheckTypes.LEVI = TRUE;
	m_ConflictCheckTypes.LIGH = TRUE;
	m_ConflictCheckTypes.LOCK = TRUE;
	m_ConflictCheckTypes.MGEF = TRUE;
	m_ConflictCheckTypes.MISC = TRUE;
	m_ConflictCheckTypes.NPC_ = TRUE;
	m_ConflictCheckTypes.PGRD = TRUE;
	m_ConflictCheckTypes.PROB = TRUE;
	m_ConflictCheckTypes.RACE = TRUE;
	m_ConflictCheckTypes.REGN = TRUE;
	m_ConflictCheckTypes.REPA = TRUE;
	m_ConflictCheckTypes.SCPT = TRUE;
	m_ConflictCheckTypes.SKIL = TRUE;
	m_ConflictCheckTypes.SNDG = TRUE;
	m_ConflictCheckTypes.SOUN = TRUE;
	m_ConflictCheckTypes.SPEL = TRUE;
	m_ConflictCheckTypes.STAT = TRUE;
	m_ConflictCheckTypes.WEAP = TRUE;

	m_UncleanCheckTypes.ACTI = TRUE;
	m_UncleanCheckTypes.ALCH = TRUE;
	m_UncleanCheckTypes.APPA = TRUE;
	m_UncleanCheckTypes.ARMO = TRUE;
	m_UncleanCheckTypes.BODY = TRUE;
	m_UncleanCheckTypes.BOOK = TRUE;
	m_UncleanCheckTypes.BSGN = TRUE;
	m_UncleanCheckTypes.CLAS = TRUE;
	m_UncleanCheckTypes.CLOT = TRUE;
	m_UncleanCheckTypes.CONT = TRUE;
	m_UncleanCheckTypes.CREA = TRUE;
	m_UncleanCheckTypes.DOOR = TRUE;
	m_UncleanCheckTypes.ENCH = TRUE;
	m_UncleanCheckTypes.FACT = TRUE;
	m_UncleanCheckTypes.GLOB = TRUE;
	m_UncleanCheckTypes.GMST = TRUE;
	m_UncleanCheckTypes.INFO = FALSE;
	m_UncleanCheckTypes.INGR = TRUE;
	m_UncleanCheckTypes.LAND = TRUE;
	m_UncleanCheckTypes.LEVC = TRUE;
	m_UncleanCheckTypes.LEVI = TRUE;
	m_UncleanCheckTypes.LIGH = TRUE;
	m_UncleanCheckTypes.LOCK = TRUE;
	m_UncleanCheckTypes.MGEF = TRUE;
	m_UncleanCheckTypes.MISC = TRUE;
	m_UncleanCheckTypes.NPC_ = TRUE;
	m_UncleanCheckTypes.PGRD = TRUE;
	m_UncleanCheckTypes.PROB = TRUE;
	m_UncleanCheckTypes.RACE = TRUE;
	m_UncleanCheckTypes.REGN = TRUE;
	m_UncleanCheckTypes.REPA = TRUE;
	m_UncleanCheckTypes.SCPT = TRUE;
	m_UncleanCheckTypes.SKIL = TRUE;
	m_UncleanCheckTypes.SNDG = TRUE;
	m_UncleanCheckTypes.SOUN = TRUE;
	m_UncleanCheckTypes.SPEL = TRUE;
	m_UncleanCheckTypes.STAT = TRUE;
	m_UncleanCheckTypes.WEAP = TRUE;

	m_ExtractOpt.bIncCELL = TRUE;
	m_ExtractOpt.bIncINFO = TRUE;
	m_ExtractOpt.bIncLAND = FALSE;
	m_ExtractOpt.bIncPGRD = FALSE;

	m_bLoadEsms = TRUE;

	m_DefOpFlags = CHECK_ALL & ~CHECK_GLOBAL;

	WIN32_FIND_DATA FindFileData;
	
	HANDLE hFind = FindFirstFile(AfxGetApp()->m_pszProfileName,&FindFileData);

	if(hFind == INVALID_HANDLE_VALUE)
	{

		HANDLE hFile = CreateFile(AfxGetApp()->m_pszProfileName,GENERIC_WRITE,FILE_SHARE_READ,NULL,
					CREATE_NEW,FILE_ATTRIBUTE_NORMAL,NULL);
		CloseHandle(hFile);

		//Write Default Data to ini File
		WritePrivateProfileStruct("Options","ConflictChecks",&m_ConflictCheckTypes,
					sizeof(m_ConflictCheckTypes),AfxGetApp()->m_pszProfileName);
		WritePrivateProfileStruct("Options","UncleanChecks",&m_UncleanCheckTypes,
					sizeof(m_UncleanCheckTypes),AfxGetApp()->m_pszProfileName);
		WritePrivateProfileStruct("Options","DefaultOps",&m_DefOpFlags,
					sizeof(m_DefOpFlags),AfxGetApp()->m_pszProfileName);
		WritePrivateProfileStruct("Options","LoadEsms",&m_bLoadEsms,
					sizeof(m_bLoadEsms),AfxGetApp()->m_pszProfileName);
		WritePrivateProfileStruct("Options","ExtractOpts",&m_ExtractOpt,
					sizeof(m_ExtractOpt),AfxGetApp()->m_pszProfileName);
		WritePrivateProfileString("DialogRepair","EspFilename",m_strDlgEspFile,
					AfxGetApp()->m_pszProfileName);
		WritePrivateProfileString("DialogRepair","EspPath",m_strDlgEspPath,
					AfxGetApp()->m_pszProfileName);
		WritePrivateProfileStruct("DialogRepair","Enable",&m_bEnableDialogRelink,
					sizeof(m_bEnableDialogRelink),AfxGetApp()->m_pszProfileName);
		
		for(int i = 0;i < m_cFindHistory; i++)
		{
			CHAR szNum[25], szFind[30] = "Find";
			itoa(i,szNum,10);
			strcat(szFind,szNum);
			
			WritePrivateProfileString("FindObj",szFind,m_strFind[i],
					AfxGetApp()->m_pszProfileName);
		}

	}
	else
	{
		FindClose(hFind);

		//Retreive Data from ini File
		if(!GetPrivateProfileStruct("Options","ConflictChecks",&m_ConflictCheckTypes,
					sizeof(m_ConflictCheckTypes),AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileStruct("Options","ConflictChecks",&m_ConflictCheckTypes,
					sizeof(m_ConflictCheckTypes),AfxGetApp()->m_pszProfileName);
		}

		if(!GetPrivateProfileStruct("Options","UncleanChecks",&m_UncleanCheckTypes,
					sizeof(m_UncleanCheckTypes),AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileStruct("Options","UncleanChecks",&m_UncleanCheckTypes,
					sizeof(m_UncleanCheckTypes),AfxGetApp()->m_pszProfileName);
		}

		if(!GetPrivateProfileStruct("Options","DefaultOps",&m_DefOpFlags,
					sizeof(m_DefOpFlags),AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileStruct("Options","DefaultOps",&m_DefOpFlags,
					sizeof(m_DefOpFlags),AfxGetApp()->m_pszProfileName);
		}

		if(!GetPrivateProfileStruct("Options","LoadEsms",&m_bLoadEsms,
					sizeof(m_bLoadEsms),AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileStruct("Options","LoadEsms",&m_bLoadEsms,
					sizeof(m_bLoadEsms),AfxGetApp()->m_pszProfileName);
		}

		if(!GetPrivateProfileStruct("Options","ExtractOpts",&m_ExtractOpt,
					sizeof(m_ExtractOpt),AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileStruct("Options","ExtractOpts",&m_ExtractOpt,
					sizeof(m_ExtractOpt),AfxGetApp()->m_pszProfileName);
		}

		if(!GetPrivateProfileString("DialogRepair","EspFilename","",
			m_strDlgEspFile.GetBuffer(MAX_PATH),MAX_PATH,AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileString("DialogRepair","EspFilename",m_strDlgEspFile,
					AfxGetApp()->m_pszProfileName);
		}
		m_strDlgEspFile.ReleaseBuffer();

		if(!GetPrivateProfileString("DialogRepair","EspPath","",
			m_strDlgEspPath.GetBuffer(MAX_PATH),MAX_PATH,AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileString("DialogRepair","EspPath",m_strDlgEspFile,
					AfxGetApp()->m_pszProfileName);
		}
		m_strDlgEspPath.ReleaseBuffer();

		if(!GetPrivateProfileStruct("DialogRepair","Enable",&m_bEnableDialogRelink,
					sizeof(m_bEnableDialogRelink),AfxGetApp()->m_pszProfileName))
		{
			WritePrivateProfileStruct("DialogRepair","Enable",&m_bEnableDialogRelink,
					sizeof(m_bEnableDialogRelink),AfxGetApp()->m_pszProfileName);
		}

		for(int i = 0;i < m_cFindHistory; i++)
		{
			CHAR szNum[25], szFind[30] = "Find";
			itoa(i,szNum,10);
			strcat(szFind,szNum);
			
			if(!GetPrivateProfileString("FindObj",szFind,"",
				m_strFind[i].GetBuffer(MAX_PATH),MAX_PATH,AfxGetApp()->m_pszProfileName))
			{
				WritePrivateProfileString("FindObj",szFind,m_strFind[i],
						AfxGetApp()->m_pszProfileName);
			}
			m_strFind[i].ReleaseBuffer();
		}

	}
}

CTESPCDDoc::~CTESPCDDoc()
{
	// Necessary Cleanup
	int tcnt,i;

	tcnt = m_aEspDataArray.GetSize();
	for(i = 0; i < tcnt;i++)
	{
		delete ((CTESFile*)m_aEspDataArray.GetAt(i));
	}
	
	m_aEspDataArray.RemoveAll();

	tcnt = m_aEsmDataArray.GetSize();
	for(i = 0; i < tcnt;i++)
	{
		delete ((CTESFile*)m_aEsmDataArray.GetAt(i));
	}
	
	m_aEsmDataArray.RemoveAll();

	//Write Data to ini File
	WritePrivateProfileStruct("Options","ConflictChecks",&m_ConflictCheckTypes,
					sizeof(m_ConflictCheckTypes),AfxGetApp()->m_pszProfileName);
	WritePrivateProfileStruct("Options","UncleanChecks",&m_UncleanCheckTypes,
					sizeof(m_UncleanCheckTypes),AfxGetApp()->m_pszProfileName);
	WritePrivateProfileStruct("Options","DefaultOps",&m_DefOpFlags,
					sizeof(m_DefOpFlags),AfxGetApp()->m_pszProfileName);
	WritePrivateProfileStruct("Options","LoadEsms",&m_bLoadEsms,
					sizeof(m_bLoadEsms),AfxGetApp()->m_pszProfileName);
	WritePrivateProfileStruct("Options","ExtractOpts",&m_ExtractOpt,
					sizeof(m_ExtractOpt),AfxGetApp()->m_pszProfileName);
	WritePrivateProfileString("DialogRepair","EspFilename",m_strDlgEspFile,
					AfxGetApp()->m_pszProfileName);
	WritePrivateProfileString("DialogRepair","EspPath",m_strDlgEspPath,
					AfxGetApp()->m_pszProfileName);
	WritePrivateProfileStruct("DialogRepair","Enable",&m_bEnableDialogRelink,
					sizeof(m_bEnableDialogRelink),AfxGetApp()->m_pszProfileName);

	for(i = 0;i < m_cFindHistory; i++)
	{
		CHAR szNum[25], szFind[30] = "Find";
		itoa(i,szNum,10);
		strcat(szFind,szNum);

		WritePrivateProfileString("FindObj",szFind,m_strFind[i],
				AfxGetApp()->m_pszProfileName);
	}

}

BOOL CTESPCDDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;
	
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CTESPCDDoc serialization

void CTESPCDDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
	}
	else
	{
	}
}

/////////////////////////////////////////////////////////////////////////////
// CTESPCDDoc diagnostics

#ifdef _DEBUG
void CTESPCDDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CTESPCDDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTESPCDDoc commands

BOOL CTESPCDDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;

	return TRUE;
}

void CTESPCDDoc::OnFileChooseplugins() 
{
	if(m_bWorkerThreadActive)
		return;

	CTreeView* pView;
	CTreeCtrl* pTreeCtrl;
	int cIterator = 0;
	int nSubLen = 0;
	int nLen = 0;
	POSITION pos1, pos2;
	CString strNewPath;
	CMapStringToOb * dataMap;

	OPENFILENAME vOpenFile;
	CString strFile;
	CString strDefExt = _T("esp");
	
	//TRACE0("DEBUG SETTING\n");
	//char*  strFilter= _T("Plugin Files (*.esm;*.esp)\0*.esm;*.esp\0All Files (*.*)\0*.*\0\0");
	char strFilter[] = _T("Plugin Files (*.esp)\0*.esp\0Master Files (*.esm)\0*.esm\0All Files (*.*)\0*.*\0\0");

	vOpenFile.lStructSize = sizeof(vOpenFile); //OPENFILENAME_SIZE_VERSION_400;
	vOpenFile.hwndOwner = AfxGetMainWnd()->GetSafeHwnd();
	vOpenFile.lpstrFilter = strFilter;
	vOpenFile.lpstrCustomFilter = NULL;
	vOpenFile.nFilterIndex = 1;
	vOpenFile.lpstrFile = strFile.GetBuffer(OPENDIALOG_BUFFERSIZE);
	vOpenFile.nMaxFile = OPENDIALOG_BUFFERSIZE;
	vOpenFile.lpstrFileTitle = NULL;
	vOpenFile.lpstrInitialDir = m_strDataFilesPath;
	vOpenFile.lpstrTitle = _T("Select Files\0");
	vOpenFile.Flags = OFN_ALLOWMULTISELECT | OFN_EXPLORER | OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
	vOpenFile.lpstrDefExt = strDefExt.GetBuffer(4);

	if(!GetOpenFileName(&vOpenFile))
		return;

	pView = (CTreeView*)GetView("CLeftView");

	pTreeCtrl = &pView->GetTreeCtrl();

	if(m_FilesRoot)
	{	//Forces Deletion of any Leafs
		pTreeCtrl->DeleteItem(m_FilesRoot);
		m_FilesRoot = pTreeCtrl->InsertItem("Plugin Files",1,1,TVI_ROOT, TVI_LAST);
	}

	strFile.ReleaseBuffer(OPENDIALOG_BUFFERSIZE);
	
	strNewPath = strFile.Left(vOpenFile.nFileOffset);

	// Add Ending Slash
	strNewPath.GetBuffer(0);
	strNewPath.ReleaseBuffer();
	if(strNewPath.Right(1) != "\\")
	{
		strNewPath += _T("\\");
	}
		
	nSubLen = vOpenFile.nFileOffset;
	cIterator = nSubLen;
	nLen = OPENDIALOG_BUFFERSIZE;
	// Add New Filenames to List
	while(cIterator < nLen && nSubLen > 1)
	{
		strFile.Delete(0,nSubLen);
		if(strFile == "")
			break;
		
		// Check to see if in 
		pos1 = m_EsmFilesList.GetHeadPosition();
		pos2 = m_EsmPathList.GetHeadPosition();
		while(pos1)
		{
			if(strFile == m_EsmFilesList.GetNext(pos1))
			{
				if(pos1)
					m_EsmFilesList.GetPrev(pos1);
				else
					pos1 = m_EsmFilesList.GetTailPosition();

				if(m_bEsmExtracted)
				{
					for(int i = 0;i < m_aEsmDataArray.GetSize();i++)
					{
						dataMap = &((CTESFile*)m_aEsmDataArray.GetAt(i))->m_EntryMap;

						if(strFile == *((CString*)(*dataMap)["FILE"]))
						{
							delete ((CTESFile *) m_aEsmDataArray.GetAt(i));
							m_aEsmDataArray.RemoveAt(i);
							break;
						}
					}

					m_EsmFilesList.RemoveAt(pos1);
					m_EsmPathList.RemoveAt(pos2);
				}
				else
				{
					m_EsmFilesList.RemoveAt(pos1);
					m_EsmPathList.RemoveAt(pos2);
				}
				break;
			}
			else
			{
				m_EsmPathList.GetNext(pos2);
			}
		}

		// If Not Already Included Add to List
		if(!m_EspFilesList.Find(strFile,NULL) && 
		   !m_EspExtractedFileList.Find(strFile,NULL) &&
		   strFile != m_EspMasterName &&
		   strFile != m_EspMasterExtractedName)
		{
			m_EspFilesList.AddHead(strFile);
			m_EspPathList.AddHead(strNewPath);
		}

		nSubLen = strFile.Find('\0') + 1;
		cIterator += nSubLen;
	}


	nLen = m_EspFilesList.GetCount();
	pos1 = m_EspFilesList.GetTailPosition();
	// Add Not Extracted Files to TreeCtrl
	for(cIterator = 0;cIterator < nLen;cIterator++)
	{
		pTreeCtrl->InsertItem(m_EspFilesList.GetPrev(pos1),0,0,m_FilesRoot);
	}
	
	nLen = m_EspExtractedFileList.GetCount();
	if(m_EspMasterExtractedName != "") // Does not Display Master
		nLen--;
	pos1 = m_EspExtractedFileList.GetHeadPosition();
	// Adds Already Extracted Files to TreeCtrl
	for(cIterator = 0;cIterator < nLen;cIterator++)
	{
		pTreeCtrl->InsertItem(m_EspExtractedFileList.GetNext(pos1),0,0,m_FilesRoot);
	}

	
	pTreeCtrl->Expand(m_FilesRoot,TVE_EXPAND);

	UpdateAllViews(NULL);
}

void CTESPCDDoc::OnFileChooseactiveplugins() 
{

	if(m_bWorkerThreadActive)
		return;
	
	CTreeView * pView = (CTreeView*)GetView("CLeftView");
	CTreeCtrl * pTreeCtrl = &pView->GetTreeCtrl();


	CString strIniName;
	CString strPluginName;
	CString strTemp;
	int bPlugins = true;

	for(int i = 0;bPlugins;i++)
	{
		strIniName.Format("GameFile%d",i);

		bPlugins = GetPrivateProfileString("Game Files",strIniName,"",strPluginName.GetBuffer(MAX_PATH),
				MAX_PATH,m_strBasePath + _T("\\Morrowind.ini"));
	
		strPluginName.ReleaseBuffer();

		strTemp = strPluginName.Right(3);
		strTemp.MakeUpper();

		if(strPluginName != "" && strTemp == "ESP")
		{
			if(!m_EspFilesList.Find(strPluginName,NULL) && 
			   !m_EspExtractedFileList.Find(strPluginName,NULL) &&
			   strPluginName != m_EspMasterName && 
			   strPluginName != m_EspMasterExtractedName)
			{
				m_EspFilesList.AddHead(strPluginName);
				m_EspPathList.AddHead(m_strDataFilesPath);
			}
		}

	}

	int length, count;
	POSITION pos;

	if(m_FilesRoot)
	{	//Forces Deletion of any Leafs
		pTreeCtrl->DeleteItem(m_FilesRoot);
		m_FilesRoot = pTreeCtrl->InsertItem("Plugin Files",1,1,TVI_ROOT, TVI_LAST);
	}

	length = m_EspFilesList.GetCount();
	pos = m_EspFilesList.GetTailPosition();
	for(count = 0;count < length;count++)
	{
		pTreeCtrl->InsertItem(m_EspFilesList.GetPrev(pos),0,0,m_FilesRoot);
	}
	
	length = m_EspExtractedFileList.GetCount();
	if(m_EspMasterExtractedName != "") //Does not Display Master
		length--;
	pos = m_EspExtractedFileList.GetHeadPosition();
	for(count = 0;count < length;count++)
	{
		pTreeCtrl->InsertItem(m_EspExtractedFileList.GetNext(pos),0,0,m_FilesRoot);
	}

	pTreeCtrl->Expand(m_FilesRoot,TVE_EXPAND);

	UpdateAllViews(NULL);

}

void CTESPCDDoc::OnFileChoosemaster() 
{
	if(m_bWorkerThreadActive)
		return;

	int tcnt;
	CTreeView* pView;
	CTreeCtrl* pTreeCtrl;
	
	OPENFILENAME vOpenFile;
	CString strFile;
	CString strDefExt = _T("esp");
	//char  strFilter[] = _T("Plugin Files (*.esp;*.zip)\0*.esp;*.zip\0All Files (*.*)\0*.*\0\0");
	char strFilter[] = _T("Plugin Files (*.esp)\0*.esp\0All Files (*.*)\0*.*\0\0");

	vOpenFile.lStructSize = sizeof(vOpenFile); //OPENFILENAME_SIZE_VERSION_400;
	vOpenFile.hwndOwner = AfxGetMainWnd()->GetSafeHwnd();
	vOpenFile.lpstrFilter = strFilter;
	vOpenFile.lpstrCustomFilter = NULL;
	vOpenFile.nFilterIndex = 1;
	vOpenFile.lpstrFile = strFile.GetBuffer(OPENDIALOG_BUFFERSIZE);
	vOpenFile.nMaxFile = OPENDIALOG_BUFFERSIZE;
	vOpenFile.lpstrFileTitle = NULL;
	vOpenFile.lpstrInitialDir = m_strDataFilesPath;
	vOpenFile.lpstrTitle = _T("Select Master Plugin to Check Against\0");
	vOpenFile.Flags = OFN_EXPLORER | OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST;
	vOpenFile.lpstrDefExt = strDefExt.GetBuffer(4);

	if(!GetOpenFileName(&vOpenFile))
		return;

	pView = (CTreeView*)GetView("CLeftView");

	pTreeCtrl = &pView->GetTreeCtrl();

	if(m_SingleMaster)
	{	//Forces Deletion of any Leafs
		pTreeCtrl->DeleteItem(m_SingleMaster);
		m_SingleMaster = pTreeCtrl->InsertItem("Master Plugin",1,1,TVI_ROOT, TVI_FIRST);
	}

	strFile.ReleaseBuffer();

	if(m_EspFilesList.Find(strFile.Mid(vOpenFile.nFileOffset)) != NULL || 
	   m_EspExtractedFileList.Find(strFile.Mid(vOpenFile.nFileOffset)) != NULL)
	{
		AfxGetMainWnd()->MessageBox("Please Remove Chosen File from Plugin List Before Selecting it as a Master.",
									"Cannot Complete Request");
		return;
	}

	if(m_EspMasterExtractedName != "")
	{
		m_EspExtractedFileList.RemoveTail();
		tcnt = m_aEspDataArray.GetSize();
		delete ((CTESFile*)m_aEspDataArray.GetAt(tcnt - 1));
		m_aEspDataArray.RemoveAt(tcnt - 1);
		m_EspMasterExtractedName = "";
	}
	
	m_EspMasterPath = strFile.Left(vOpenFile.nFileOffset);
	m_EspMasterName = strFile.Mid(vOpenFile.nFileOffset);

	if(m_EspMasterPath.Right(1) != "\\")
	{
		m_EspMasterPath += _T("\\");
	}

	pTreeCtrl->InsertItem(m_EspMasterName,0,0,m_SingleMaster);

	pTreeCtrl->Expand(m_SingleMaster,TVE_EXPAND);

	UpdateAllViews(NULL);
}

void CTESPCDDoc::OnUpdateFileChoosemaster(CCmdUI* pCmdUI) 
{
	
	if(m_nOperationType == SINGLEVERSUS)
		pCmdUI->Enable(true);
	else
		pCmdUI->Enable(false);
	
}

CView* CTESPCDDoc::GetView(const CString classname)
{
	CView* pView;
	POSITION pos = GetFirstViewPosition();
	do
	{
		if(pos == NULL)
			return NULL;
		pView = GetNextView(pos);
		
	}while (pView->GetRuntimeClass()->m_lpszClassName != classname); 

	return pView;
}

void CTESPCDDoc::OnFileSave() 
{
	if(m_bWorkerThreadActive)
		return;

	CConflictView *pCView = (CConflictView*)GetView("CConflictView");
	CWarningView *pWView = (CWarningView*)GetView("CWarningView");
	CEditorView *pEView = (CEditorView*)GetView("CEditorView");

	CString strFile;
	CString strFiletitle = "TESPCDlog.txt";
	OPENFILENAME vOpenFile;
	CString strDefExt = _T("txt");
	char strFilter[] = _T("Any File (*.*)\0*.*\0\0");
	vOpenFile.lStructSize = sizeof(vOpenFile); //OPENFILENAME_SIZE_VERSION_400;
	vOpenFile.hwndOwner = AfxGetMainWnd()->GetSafeHwnd();
	vOpenFile.lpstrFilter = strFilter;
	vOpenFile.lpstrCustomFilter = NULL;
	vOpenFile.nFilterIndex = 0;
	vOpenFile.lpstrFile = strFile.GetBuffer(1000);
	vOpenFile.nMaxFile = 1000;
	vOpenFile.lpstrFileTitle = NULL;
	vOpenFile.lpstrInitialDir = m_strDataFilesPath;
	vOpenFile.lpstrTitle = _T("Save Log As\0");
	vOpenFile.Flags = OFN_EXPLORER | OFN_HIDEREADONLY |
					  OFN_OVERWRITEPROMPT | OFN_PATHMUSTEXIST | OFN_EXTENSIONDIFFERENT;
	vOpenFile.lpstrDefExt = strDefExt.GetBuffer(4);

	if(!GetSaveFileName(&vOpenFile))
	{
		if(CommDlgExtendedError() != 0)
			MessageBox(AfxGetMainWnd()->GetSafeHwnd(),"Error Selecting Save File", "Error", MB_OK);

		return;
	}
	strFile.ReleaseBuffer();
	WIN32_FIND_DATA vfinddata;
	HANDLE hFind;
	hFind = FindFirstFile(strFile,&vfinddata);

	if(hFind != INVALID_HANDLE_VALUE)
	{
		DeleteFile(strFile);
		FindClose(hFind);
	}

	pCView->m_pGridCtrl->Save(strFile);
	pWView->m_pGridCtrl->Save(strFile);
	pEView->m_pGridCtrl->Save(strFile);
}


/////////// Friend Worker Thread Entry Point //////////////////////////////
unsigned int ThreadFunc(LPVOID pDoc)
{
	
	((CTESPCDDoc*)pDoc)->m_bWorkerThreadActive = TRUE;
		
	SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,0,0);

	switch(((CTESPCDDoc*)pDoc)->m_nThreadOperation)
	{
	case 0:
		((CTESPCDDoc*)pDoc)->BeginExtract();
		break;
	case 1:
		((CTESPCDDoc*)pDoc)->BeginCheck();
		break;
	case 2:
		((CTESPCDDoc*)pDoc)->BeginExtract();
		((CTESPCDDoc*)pDoc)->BeginCheck();
		break;
	case 3:
		((CTESPCDDoc*)pDoc)->BeginAutoClean();
		break;
	case 4:
		((CTESPCDDoc*)pDoc)->BeginExtract();
		((CTESPCDDoc*)pDoc)->BeginAutoClean();
		break;
	case 5:
		((CTESPCDDoc*)pDoc)->BeginFind();
		break;
	case 6:
		((CTESPCDDoc*)pDoc)->BeginExtract();
		((CTESPCDDoc*)pDoc)->BeginFind();
		break;
	case 7:
		((CTESPCDDoc*)pDoc)->BeginFindGlobalChanges();
		break;
	case 8:
		((CTESPCDDoc*)pDoc)->BeginExtract();
		((CTESPCDDoc*)pDoc)->BeginFindGlobalChanges();
		break;
	case 9:
		((CTESPCDDoc*)pDoc)->BeginEditor();
		break;
	case 10:
		((CTESPCDDoc*)pDoc)->BeginExtract();
		((CTESPCDDoc*)pDoc)->BeginEditor();
		break;
	case 11:
		((CTESPCDDoc*)pDoc)->BeginDialogRepair();
		break;
	default:
		AfxGetMainWnd()->MessageBox("Operation Does Not Exist","Warning",MB_ICONEXCLAMATION | MB_OK);
		break;
	}

	((CTESPCDDoc*)pDoc)->m_bWorkerThreadActive = false;
	return 0;
}
///////////////////////////////////////////////////////////////////////////


void CTESPCDDoc::OnOperationsExtractdata() 
{
	if(!m_bWorkerThreadActive)
	{
		if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "" || !m_bEsmExtracted)
		{
			((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

			m_nThreadOperation = 0;	
			m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
			m_hWorkerThread = m_pWorkerThread->m_hThread;
		}
	}
}

void CTESPCDDoc::OnOperationsAutoclean() 
{
	if(!m_bWorkerThreadActive)
	{

		if(m_nOperationType == SINGLEVERSUS && m_EspMasterName == "" && m_EspMasterExtractedName == "")
			return;

		((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

		if(m_nOperationType == SINGLEVERSUS)
		{
			if(m_EspFilesList.GetCount() == 0 && m_EspMasterExtractedName != "")
			{
				m_nThreadOperation = 3;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
			else if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "")
			{
				m_nThreadOperation = 4;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
		}
		else
		{
			if(m_EspFilesList.GetCount() == 0 && m_EspExtractedFileList.GetCount() > 0)
			{
				m_nThreadOperation = 3;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
			else if(m_EspFilesList.GetCount() > 0)
			{
				m_nThreadOperation = 4;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
			
		}
	}

}

void CTESPCDDoc::PrepareCheck()
{
	if(!m_bWorkerThreadActive)
	{

		if(m_nOperationType == SINGLEVERSUS && m_EspMasterName == "" && m_EspMasterExtractedName == "")
		{
			AfxGetMainWnd()->MessageBox("Select a Master Plugin for Single Versus Mode",
										"Cannot Continue",MB_OK | MB_ICONINFORMATION);
			return;
		}

		((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

		if(m_nOperationType == SINGLEVERSUS)
		{
			if(m_EspFilesList.GetCount() == 0 && m_EspMasterExtractedName != "")
			{
				m_nThreadOperation = 1;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
			else if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "")
			{
				m_nThreadOperation = 2;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
		}
		else
		{
			if(m_EspFilesList.GetCount() == 0 && m_EspExtractedFileList.GetCount() > 0)
			{
				m_nThreadOperation = 1;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
			else if(m_EspFilesList.GetCount() > 0)
			{
				m_nThreadOperation = 2;
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}	
		}
	}

}

void CTESPCDDoc::OnOperationsAutorepairdialog() 
{
	m_bRepairAuto = TRUE;

	if(!m_bWorkerThreadActive)
	{
		m_nThreadOperation = 11;	
		m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
		m_hWorkerThread = m_pWorkerThread->m_hThread;
	}
	
}

void CTESPCDDoc::OnOperationsRepairdialog() 
{
	m_bRepairAuto = FALSE;

	if(!m_bWorkerThreadActive)
	{
		m_nThreadOperation = 11;	
		m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
		m_hWorkerThread = m_pWorkerThread->m_hThread;
	}

}


void CTESPCDDoc::OnOperationsViewallespdata() 
{
	if(!m_bWorkerThreadActive)
	{

		((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

		m_iViewDataMode = ESPVIEW;
		m_strEditorFile = "";

		if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "")
		{
			m_nThreadOperation = 10;	
			m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
			m_hWorkerThread = m_pWorkerThread->m_hThread;
		}
		else
		{
			m_nThreadOperation = 9;	
			m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
			m_hWorkerThread = m_pWorkerThread->m_hThread;
		}

	}
}

void CTESPCDDoc::OnOperationsViewallesmdata() 
{
	if(!m_bWorkerThreadActive)
	{

		((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

		m_iViewDataMode = ESMVIEW;
		m_strEditorFile = "";

		if(!m_bEsmExtracted && m_bLoadEsms)
		{
			m_nThreadOperation = 10;	
			m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
			m_hWorkerThread = m_pWorkerThread->m_hThread;
		}
		else
		{
			m_nThreadOperation = 9;	
			m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
			m_hWorkerThread = m_pWorkerThread->m_hThread;
		}

	}
}


void CTESPCDDoc::OnOperationsCheckconflicts() 
{
	m_CurOpFlags = CHECK_CONFLICTS;
	PrepareCheck();
}

void CTESPCDDoc::OnOperationsCheckall() 
{
	m_CurOpFlags = CHECK_ALL;
	PrepareCheck();
}

void CTESPCDDoc::OnOperationsCheckdeleted() 
{
	m_CurOpFlags = CHECK_DELETIONS;
	PrepareCheck();
}

void CTESPCDDoc::OnOperationsCheckunclean() 
{
	m_CurOpFlags = CHECK_UNCLEAN;
	PrepareCheck();
	
}

void CTESPCDDoc::OnOperationsCheckAllGlobalChanges() 
{
	if(!m_bWorkerThreadActive)
	{
		((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

		m_strGlobCheckFile.Empty();

		if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "")
		{
			m_nThreadOperation = 8;	
			m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
			m_hWorkerThread = m_pWorkerThread->m_hThread;
		}
		else
		{
			m_nThreadOperation = 7;	
			m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
			m_hWorkerThread = m_pWorkerThread->m_hThread;
		}

	}
}

void CTESPCDDoc::OnOperationsViewfiledata() 
{
	CViewFileDlg dlgVFile;
	POSITION pos;

	if(!m_bWorkerThreadActive)
	{
		pos = m_EspFilesList.GetHeadPosition();

		while(pos != NULL)
		{
			dlgVFile.AddEspFile((CString)m_EspFilesList.GetNext(pos));
		}

		pos = m_EspExtractedFileList.GetHeadPosition();

		while(pos != NULL)
		{
			dlgVFile.AddEspFile((CString)m_EspExtractedFileList.GetNext(pos));
		}

		if(m_nOperationType == SINGLEVERSUS)
		{
			if(m_EspMasterName != "")
			{
				dlgVFile.AddEspFile(m_EspMasterName);
			}
			else if(m_EspMasterExtractedName != "")
			{
				dlgVFile.AddEspFile(m_EspMasterExtractedName);
			}
		}

		GetESMPaths();

		pos = m_EsmFilesList.GetHeadPosition();

		while(pos != NULL)
		{
			dlgVFile.AddEsmFile((CString)m_EsmFilesList.GetNext(pos));
		}

		if(dlgVFile.DoModal() == IDOK)
		{
			((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

			if(dlgVFile.m_iFileType == 0)
			{
				m_iViewDataMode = ESPVIEW;
			}
			else
			{
				m_iViewDataMode = ESMVIEW;
			}
			
			if(dlgVFile.m_iViewMode == 0)
			{
				m_strGlobCheckFile = dlgVFile.m_strChosenFile;

				if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "")
				{
					m_nThreadOperation = 8;	
					m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
					m_hWorkerThread = m_pWorkerThread->m_hThread;
				}
				else
				{
					m_nThreadOperation = 7;	
					m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
					m_hWorkerThread = m_pWorkerThread->m_hThread;
				}
			}
			else
			{
				m_strEditorFile = dlgVFile.m_strChosenFile;
			
				if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "" || (!m_bEsmExtracted && m_bLoadEsms))
				{
					m_nThreadOperation = 10;	
					m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
					m_hWorkerThread = m_pWorkerThread->m_hThread;
				}
				else
				{
					m_nThreadOperation = 9;	
					m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
					m_hWorkerThread = m_pWorkerThread->m_hThread;
				}
			}
		}
	}
}

void CTESPCDDoc::OnOperationsFindobject() 
{
	
	if(!m_bWorkerThreadActive)
	{
		CFindObject dlgFindObj;

		if(dlgFindObj.DoModal() == IDOK)
		{
			m_bFOMatchcase = dlgFindObj.m_bMatchcase;
			m_bFOMatchword = dlgFindObj.m_bMatchword;
			m_bFOLookdata = dlgFindObj.m_bLookdata;
			m_bFOSearchEsm = (dlgFindObj.m_iSearchMode == 1);
			m_strFOObjName = dlgFindObj.m_strObjName;

			((CMainFrame*)AfxGetMainWnd())->ClosePopupWindows();

			if(m_EspFilesList.GetCount() > 0 || m_EspMasterName != "")
			{
				m_nThreadOperation = 6;	
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
			else
			{
				m_nThreadOperation = 5;	
				m_pWorkerThread = AfxBeginThread(ThreadFunc,this);
				m_hWorkerThread = m_pWorkerThread->m_hThread;
			}
		}
	}
}


BOOL CTESPCDDoc::GetESMPaths()
{
	// Paths Loaded Only if EsmList is Empty
	if(m_EsmFilesList.GetCount() > 0)
		return TRUE;

	HANDLE hFind;
	WIN32_FIND_DATA findData;

	m_EsmFilesList.RemoveAll();
	m_EsmPathList.RemoveAll();

	hFind = FindFirstFile(m_strDataFilesPath + "*.esm",&findData);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		if(!m_EspFilesList.Find(findData.cFileName,NULL) &&
		   !m_EspExtractedFileList.Find(findData.cFileName,NULL))
		{
			m_EsmFilesList.AddHead(findData.cFileName);
			m_EsmPathList.AddHead(m_strDataFilesPath);
		}

		while(FindNextFile(hFind,&findData))
		{
			if(!m_EspFilesList.Find(findData.cFileName,NULL) &&
		       !m_EspExtractedFileList.Find(findData.cFileName,NULL))
			{
				m_EsmFilesList.AddHead(findData.cFileName);
				m_EsmPathList.AddHead(m_strDataFilesPath);
			}
		}
			
		FindClose(hFind);

		if(m_EsmFilesList.GetCount() <= 0)
			return FALSE;
		else
			return TRUE;
	}
	else
	{
		return FALSE;
	}
}

void CTESPCDDoc::BeginExtract()
{
	CTESFile* pdynEsp;
	CPtrArray esparray;
	POSITION pos1,pos2;
	CString barstr, sFilename, sPath;
	int MaxCnt, i;


	HWND hWnd = AfxGetMainWnd()->GetSafeHwnd();

	try
	{
		i = 0;

		// Extact Master Esm Data if we haven't Already Done So.

		if(m_bLoadEsms && !m_bEsmExtracted)
		{
			if(GetESMPaths())
			{
				pos1 = m_EsmFilesList.GetHeadPosition();
				pos2 = m_EsmPathList.GetHeadPosition();

				MaxCnt = m_EsmFilesList.GetCount();

				barstr = "Extracting Master Esm Data 0% complete";
				if(!m_bWorkerThreadForceExit)
					SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,(UINT)MaxCnt);

				do
				{// Extract Data From Plugins
					pdynEsp = new CTESFile;
				
					sFilename = m_EsmFilesList.GetNext(pos1);
					sPath = m_EsmPathList.GetNext(pos2);
					if(pdynEsp->OpenFile(sFilename,sPath))
					{
						//m_EsmExtractedFileList.AddHead(sFilename);
						//m_EsmExtractedPathList.AddHead(sPath);

						//Starts Parser Extraction Function
						if(pdynEsp->ExtractData(m_bWorkerThreadForceExit, &m_ExtractOpt))
						{
							pdynEsp->CloseFile();				
							//Saves pointer before dynamically overwritten
							m_aEsmDataArray.InsertAt(0,pdynEsp); //Adds to Head of Array
						}
						else
						{
							delete pdynEsp;
						}
					}
					else
					{
						delete pdynEsp;
					}

					i++;
					barstr.Format("Extracting Master Esm Data %d%% complete", i*100/MaxCnt);
					if(!m_bWorkerThreadForceExit)
						SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,(UINT)MaxCnt);
					
				}while(pos1 != NULL && pos2 != NULL && !m_bWorkerThreadForceExit);
				
				m_bEsmExtracted = TRUE;

				if(!m_bWorkerThreadForceExit)
					SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,0);
			}

		}

		i = 0;
		MaxCnt = m_EspFilesList.GetCount();

		// Extract Data From Master Esp Plugin

		if(m_nOperationType == SINGLEVERSUS && m_EspMasterName != "" && m_EspMasterExtractedName == "")
		{// Extract Data from Master Plugin
			MaxCnt++;
			barstr = "Extracting Plugin Data 0% complete";
			if(!m_bWorkerThreadForceExit)
				SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,(UINT)MaxCnt);

			pdynEsp = new CTESFile;
			if(pdynEsp->OpenFile(m_EspMasterName,m_EspMasterPath))
			{
				m_EspExtractedFileList.AddTail(m_EspMasterName);
				m_EspExtractedPathList.AddTail(m_EspMasterPath);
				//Starts Parser Extraction Function
				if(pdynEsp->ExtractData(m_bWorkerThreadForceExit, &m_ExtractOpt))
				{
					pdynEsp->CloseFile();				
					//Saves pointer before dynamically overwritten
					m_aEspDataArray.Add(pdynEsp); //Adds to End of Array
				}
				else
				{
					delete pdynEsp;
				}
			}
			else
			{
				delete pdynEsp;
			}
			i++;
			barstr.Format("Extracting Plugin Data %d%% complete", i*100/MaxCnt);
			if(!m_bWorkerThreadForceExit)
				SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,(UINT)MaxCnt);

			m_EspMasterExtractedName = m_EspMasterName;
			m_EspMasterExtractedPath = m_EspMasterPath;
			m_EspMasterName = "";
			m_EspMasterPath = "";
		}
		else
		{
			barstr = "Extracting Plugin Data 0% complete";
			if(!m_bWorkerThreadForceExit)
				SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,(UINT)MaxCnt);
		}
		
		// Extract Data From Rest of Plugins

		if(m_EspFilesList.GetCount() > 0)
		{
			pos1 = m_EspFilesList.GetHeadPosition();
			pos2 = m_EspPathList.GetHeadPosition();

			do
			{// Extract Data From Plugins
				pdynEsp = new CTESFile;
			
				sFilename = m_EspFilesList.GetNext(pos1);
				sPath = m_EspPathList.GetNext(pos2);
				if(pdynEsp->OpenFile(sFilename,sPath))
				{
					m_EspExtractedFileList.AddHead(sFilename);
					m_EspExtractedPathList.AddHead(sPath);
					//Starts Parser Extraction Function
					if(pdynEsp->ExtractData(m_bWorkerThreadForceExit, &m_ExtractOpt))
					{
						pdynEsp->CloseFile();				
						//Saves pointer before dynamically overwritten
						m_aEspDataArray.InsertAt(0,pdynEsp); //Adds to Head of Array
					}
					else
					{
						delete pdynEsp;
					}
				}
				else
				{
					delete pdynEsp;
				}

				i++;
				barstr.Format("Extracting Plugin Data %d%% complete", i*100/MaxCnt);
				if(!m_bWorkerThreadForceExit)
					SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,(UINT)MaxCnt);
				
			}while(pos1 != NULL && pos2 != NULL && !m_bWorkerThreadForceExit);		

			m_EspFilesList.RemoveAll();
			m_EspPathList.RemoveAll();

			if(!m_bWorkerThreadForceExit)
				SendMessage(hWnd,UDM_STEPPBAR,(UINT)&barstr,0);

		}

	}
	catch(CException* err)
	{
		MessageBox(AfxGetMainWnd()->GetSafeHwnd(),"File Error","Error",MB_OK);
		err->Delete();
	}
	
}

void CTESPCDDoc::BeginAutoClean()
{

	int tcnt, i, cUncleanList;
	CTESOperations OpClass;
	CMapStringToOb *ConflictData;
	CPtrArray * uncleanList;
	CString espFileName, espEntryName, espID;
	CTESFile *pEspFile;
	CTESEntry * pDeleteEntry;
	CString barstr;
	CString strTemp;
	CConflictView* pCView = (CConflictView*) GetView("CConflictView");
	CWarningView * pWView = (CWarningView*) GetView("CWarningView");

	strTemp = "This Operation will Delete all Unclean Entries.\n";
	strTemp += "WARNING: There are a few rare cases where the Unclean entries are actually necessary.\n";
	strTemp += "Morrowind will report this when it is loading the file as Reference Missing errors.\n";
	strTemp += "This usually occurs when the objects are used with a Trap.\n";
	strTemp += "Restore from Affected Esp from Backup if this occurs.\n";

	if(MessageBox(AfxGetMainWnd()->GetSafeHwnd(),strTemp,
		"Auto Clean Plugins",MB_YESNO | MB_ICONQUESTION) == IDNO)
	{
		return;
	}
	


	tcnt = m_aEspDataArray.GetSize() ;
	
	for(i=tcnt-1;i>=0;i--)
	{
		OpClass.AddEsp(&((CTESFile*)m_aEspDataArray.GetAt(i))->m_EntryMap);
	}

	tcnt = m_aEsmDataArray.GetSize() ;

	for(i=tcnt-1;i>=0;i--)
	{
		OpClass.AddEsm(&((CTESFile*)m_aEsmDataArray.GetAt(i))->m_EntryMap);
	}

	// The Main Conflict Check Call
	ConflictData = OpClass.CheckErrors(m_bWorkerThreadForceExit, 
									  FULLCHECK, 
									  & m_ConflictCheckTypes,
									  & m_UncleanCheckTypes,
									  CHECK_UNCLEAN);

	if(!m_bWorkerThreadForceExit)
	{
	
		ConflictData->Lookup("UNCL",(CObject*&)uncleanList);

		cUncleanList = uncleanList->GetSize();

		barstr = "Auto-Deleting 0% complete";
		if(!m_bWorkerThreadForceExit)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cUncleanList);
		
		for(i = 0;i < cUncleanList && !m_bWorkerThreadForceExit;i++)
		{
			espFileName = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->Filename;
			espEntryName = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->Name;
			espID = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->ID;

			if(GetTesFileObj(espFileName,pEspFile,true))
			{

				if(GetEntry(espFileName,espEntryName,espID,pDeleteEntry,true))
				{
					if(pEspFile->DeleteEntry(*pDeleteEntry,FALSE))
					{
						pEspFile->RemoveEntry(pDeleteEntry,espID);
					}
					else
					{
						AfxMessageBox(espFileName + " - " + espEntryName + 
								  " Error Deleting Entry.",MB_OK);
					}
				}
				else
				{
					AfxMessageBox(espFileName + " - " + espEntryName + 
								  " Already Deleted or Missing.",MB_OK);
				}
			}
			else
			{
				AfxMessageBox(espFileName + " not Found.",MB_OK);
			}
		
			barstr.Format("Auto-Deleting %d%% complete", i*100/cUncleanList);
			if(!m_bWorkerThreadForceExit)
				SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)cUncleanList);

		}

		pWView->m_pGridCtrl->DeleteNonFixedRows();
		pCView->m_pGridCtrl->DeleteNonFixedRows();

		if(!m_bWorkerThreadForceExit)
		{
			//Destroy the Bar
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,0);
		}
	}

}


void CTESPCDDoc::BeginCheck()
{

	int tcnt, i, nCurRow, cUncleanList, cDeletedList;
	CTESOperations OpClass;
	CMapStringToOb *ConflictData;
	CPtrArray * conflictsList, * deletedList, * uncleanList;
	CConflictView* pCView;
	CWarningView* pWView;
	GV_ITEM Item;

	tcnt = m_aEspDataArray.GetSize() ;
	
	for(i=tcnt-1;i>=0;i--)
	{
		OpClass.AddEsp(&((CTESFile*)m_aEspDataArray.GetAt(i))->m_EntryMap);
	}

	tcnt = m_aEsmDataArray.GetSize() ;

	for(i=tcnt-1;i>=0;i--)
	{
		OpClass.AddEsm(&((CTESFile*)m_aEsmDataArray.GetAt(i))->m_EntryMap);
	}

	
	// The Main Conflict Check Call
	ConflictData = OpClass.CheckErrors(m_bWorkerThreadForceExit,
									  m_nOperationType, 
									  & m_ConflictCheckTypes,
									  & m_UncleanCheckTypes,
									  (m_CurOpFlags ? m_CurOpFlags : m_DefOpFlags));

	if(!m_bWorkerThreadForceExit)
	{
		ConflictData->Lookup("CONF",(CObject*&)conflictsList);
		ConflictData->Lookup("UNCL",(CObject*&)uncleanList);
		ConflictData->Lookup("DELE",(CObject*&)deletedList);

		tcnt = conflictsList->GetSize();
		
		pCView = (CConflictView*)GetView("CConflictView");

		Item.mask = GVIF_TEXT|GVIF_FORMAT;
		Item.nFormat = DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX;

		pCView->m_pGridCtrl->DeleteNonFixedRows();

		pCView->m_pGridCtrl->SetRowCount(tcnt+1);
		
		nCurRow = 1;
		for (i = 0; i < tcnt && !m_bWorkerThreadForceExit; i++, nCurRow++)
		{
			Item.row = nCurRow;
			
			Item.col = 0;
			Item.strText = ((CTESOperations::_sConflict*)conflictsList->GetAt(i))->ID;
			pCView->m_pGridCtrl->SetItem(&Item);

			Item.col = 1;
			Item.strText = ((CTESOperations::_sConflict*)conflictsList->GetAt(i))->Description;
			pCView->m_pGridCtrl->SetItem(&Item);

			Item.col = 2;
			Item.strText = ((CTESOperations::_sConflict*)conflictsList->GetAt(i))->Name;
			pCView->m_pGridCtrl->SetItem(&Item);

			Item.col = 3;
			Item.strText = ((CTESOperations::_sConflict*)conflictsList->GetAt(i))->Filename1;
			pCView->m_pGridCtrl->SetItem(&Item);

			Item.col = 4;
			Item.strText = ((CTESOperations::_sConflict*)conflictsList->GetAt(i))->Filename2;	
			pCView->m_pGridCtrl->SetItem(&Item);

			Item.col = 5;
			Item.strText = ((CTESOperations::_sConflict*)conflictsList->GetAt(i))->Filename3;	
			pCView->m_pGridCtrl->SetItem(&Item);
		}

		pCView->m_pGridCtrl->SortItems(0, TRUE);
		pCView->m_pGridCtrl->Refresh();

		if(tcnt > 0)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELON,1,0);
		else
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,1,0);

		//--------------------------------------------------------------

		cDeletedList = deletedList->GetSize();
		cUncleanList = uncleanList->GetSize();
		tcnt = cDeletedList + cUncleanList;
		
		pWView = (CWarningView*)GetView("CWarningView");

		pWView->m_pGridCtrl->DeleteNonFixedRows();

		pWView->m_pGridCtrl->SetRowCount(tcnt+1);

		nCurRow = 1;
		for (i = 0; i < cDeletedList && !m_bWorkerThreadForceExit; i++, nCurRow++)
		{
			Item.row = nCurRow;
			
			Item.col = 0;
			Item.strText = ((CTESOperations::_sWarning*)deletedList->GetAt(i))->ID;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 1;
			Item.strText = ((CTESOperations::_sWarning*)deletedList->GetAt(i))->Description;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 2;
			Item.strText = ((CTESOperations::_sWarning*)deletedList->GetAt(i))->Name;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 3;
			Item.strText = ((CTESOperations::_sWarning*)deletedList->GetAt(i))->Filename;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 4;
			Item.strText = ((CTESOperations::_sWarning*)deletedList->GetAt(i))->OrigFile;
			pWView->m_pGridCtrl->SetItem(&Item);
		}
		for (i = 0; i < cUncleanList && !m_bWorkerThreadForceExit; i++, nCurRow++)
		{
			Item.row = nCurRow;
			
			Item.col = 0;
			Item.strText = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->ID;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 1;
			Item.strText = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->Description;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 2;
			Item.strText = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->Name;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 3;
			Item.strText = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->Filename;
			pWView->m_pGridCtrl->SetItem(&Item);

			Item.col = 4;
			Item.strText = ((CTESOperations::_sWarning*)uncleanList->GetAt(i))->OrigFile;
			pWView->m_pGridCtrl->SetItem(&Item);
		}

		pWView->m_pGridCtrl->SortItems(0, TRUE);
		pWView->m_pGridCtrl->Refresh();
	
		if(tcnt > 0)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELON,2,0);
		else
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,2,0);

		//--------------------------------------------------------------
	}

	m_CurOpFlags = NULL;
}

void CTESPCDDoc::BeginFindGlobalChanges()
{
	int tcnt, i, nCurRow, cGlobalList;
	CTESOperations OpClass;
	CMapStringToOb * DataMap;
	CPtrArray * globalList;
	CEditorView* pEView;
	CMapStringToOb * pCheckMap;
	GV_ITEM Item;

	tcnt = m_aEspDataArray.GetSize() ;
	
	for(i=tcnt-1;i>=0;i--)
	{
		pCheckMap = &((CTESFile *) m_aEspDataArray.GetAt(i))->m_EntryMap;
		if(m_strGlobCheckFile == "" || m_strGlobCheckFile == *(CString *)(*pCheckMap)["FILE"])
		{
			OpClass.AddEsp(pCheckMap);
		}
	}

	tcnt = m_aEsmDataArray.GetSize();

	for(i=tcnt-1;i>=0;i--)
	{
		OpClass.AddEsm(&((CTESFile*)m_aEsmDataArray.GetAt(i))->m_EntryMap);
	}

	
	// The Main Global Check Call
	DataMap = OpClass.CheckErrors(m_bWorkerThreadForceExit,
									  m_nOperationType, 
									  & m_ConflictCheckTypes,
									  & m_UncleanCheckTypes,
									  CHECK_GLOBAL);

	if(!m_bWorkerThreadForceExit)
	{
		DataMap->Lookup("GCHG",(CObject*&)globalList);

		cGlobalList = globalList->GetSize();
		tcnt = cGlobalList;

		Item.mask = GVIF_TEXT|GVIF_FORMAT;
		Item.nFormat = DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX;

		pEView = (CEditorView*)GetView("CEditorView");

		pEView->m_pGridCtrl->DeleteNonFixedRows();

		pEView->m_pGridCtrl->SetRowCount(tcnt+1);

		nCurRow = 1;
		for (i = 0; i < cGlobalList && !m_bWorkerThreadForceExit; i++, nCurRow++)
		{
			Item.row = nCurRow;
			
			Item.col = 0;
			Item.strText = ((CTESOperations::_sWarning*)globalList->GetAt(i))->ID;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 1;
			Item.strText = ((CTESOperations::_sWarning*)globalList->GetAt(i))->Description;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 2;
			Item.strText = ((CTESOperations::_sWarning*)globalList->GetAt(i))->Name;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 3;
			Item.strText = ((CTESOperations::_sWarning*)globalList->GetAt(i))->Filename;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 4;
			Item.strText = ((CTESOperations::_sWarning*)globalList->GetAt(i))->OrigFile;
			pEView->m_pGridCtrl->SetItem(&Item);
		}

		pEView->m_pGridCtrl->SortItems(0, TRUE);
		pEView->m_pGridCtrl->Refresh();

		if(tcnt > 0)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELON,3,0);
		else
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,3,0);

	}
}


void CTESPCDDoc::BeginFind()
{
	int tcnt, i, nCurRow, cFindList;
	CTESOperations OpClass;
	CPtrArray * findList;
	CEditorView * pEView;
	GV_ITEM Item;

	if(m_EspExtractedFileList.GetCount() == 0)
	{
		return;
	}

	tcnt = m_aEspDataArray.GetSize() ;
	
	for(i=tcnt-1;i>=0;i--)
	{
		OpClass.AddEsp(&((CTESFile*)m_aEspDataArray.GetAt(i))->m_EntryMap);
	}

	tcnt = m_aEsmDataArray.GetSize() ;

	for(i=tcnt-1;i>=0;i--)
	{
		OpClass.AddEsm(&((CTESFile*)m_aEsmDataArray.GetAt(i))->m_EntryMap);
	}

	findList = OpClass.FindObjects(m_bWorkerThreadForceExit,
									  m_strFOObjName, 
									  m_bFOMatchcase,
									  m_bFOMatchword,
									  m_bFOLookdata,
									  m_bFOSearchEsm);

	if(!m_bWorkerThreadForceExit)
	{
		cFindList = findList->GetSize();
		tcnt = cFindList;

		if(cFindList == 0)
			AfxGetMainWnd()->MessageBox("Cannot find " + m_strFOObjName,"Not Found",MB_OK);

		Item.mask = GVIF_TEXT|GVIF_FORMAT;
		Item.nFormat = DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX;

		pEView = (CEditorView *) GetView("CEditorView");

		pEView->m_pGridCtrl->DeleteNonFixedRows();

		pEView->m_pGridCtrl->SetRowCount(tcnt+1);

		nCurRow = 1;
		for (i = 0; i < cFindList && !m_bWorkerThreadForceExit; i++, nCurRow++)
		{
			Item.row = nCurRow;
			
			Item.col = 0;
			Item.strText = ((CTESOperations::_sEditRow*)findList->GetAt(i))->ID;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 1;
			Item.strText = ((CTESOperations::_sEditRow*)findList->GetAt(i))->Description;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 2;
			Item.strText = ((CTESOperations::_sEditRow*)findList->GetAt(i))->Name;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 3;
			Item.strText = ((CTESOperations::_sEditRow*)findList->GetAt(i))->Filename;
			pEView->m_pGridCtrl->SetItem(&Item);

			Item.col = 4;
			Item.strText = ((CTESOperations::_sEditRow*)findList->GetAt(i))->OrigFile;
			pEView->m_pGridCtrl->SetItem(&Item);
		}

		pEView->m_pGridCtrl->SortItems(0, TRUE);
		pEView->m_pGridCtrl->Refresh();

		if(tcnt > 0)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELON,3,0);
		else
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,3,0);

	}
}


void CTESPCDDoc::BeginDialogRepair()
{

	CTESDialogOp dialOp;
	CRepairToolDlg dlgRepTool;
	CStringList * pFilesList, *pPathsList;
	POSITION posFiles ,posPaths;

	if(!m_bRepairAuto || m_strDlgEspFile == "" || 
		(m_EspFilesList.GetCount() == 0 && m_EspExtractedFileList.GetCount() == 0))
	{
		dlgRepTool.SetData(m_strDlgEspFile,m_strDlgEspPath, this);

		if(dlgRepTool.DoModal() != IDOK)
			return;

		pFilesList = dlgRepTool.GetEspFileList();
		pPathsList = dlgRepTool.GetEspPathList();
		dlgRepTool.GetRepairPlugin(m_strDlgEspFile,m_strDlgEspPath);

		posFiles = pFilesList->GetHeadPosition();
		posPaths = pPathsList->GetHeadPosition();

		while(posFiles != NULL)
		{
			dialOp.AddEspFile(pFilesList->GetNext(posFiles),pPathsList->GetNext(posPaths));
		}

	}
	else
	{
		pFilesList = &m_EspFilesList;
		pPathsList = &m_EspPathList;

		posFiles = pFilesList->GetHeadPosition();
		posPaths = pPathsList->GetHeadPosition();

		while(posFiles != NULL)
		{
			dialOp.AddEspFile(pFilesList->GetNext(posFiles),pPathsList->GetNext(posPaths));
		}

		pFilesList = &m_EspExtractedFileList;
		pPathsList = &m_EspExtractedPathList;

		posFiles = pFilesList->GetHeadPosition();
		posPaths = pPathsList->GetHeadPosition();

		while(posFiles != NULL)
		{
			dialOp.AddEspFile(pFilesList->GetNext(posFiles),pPathsList->GetNext(posPaths));
		}

		if(m_nOperationType == SINGLEVERSUS && 
			(m_EspMasterName != "" || m_EspMasterExtractedName != ""))
		{
			if(m_EspMasterName != "")
				dialOp.AddEspFile(m_EspMasterName,m_EspMasterPath);
			else if(m_EspMasterExtractedName != "")
				dialOp.AddEspFile(m_EspMasterExtractedName,m_EspMasterExtractedPath);

		}

	}

	GetESMPaths();

	posFiles = m_EsmFilesList.GetHeadPosition();
	posPaths = m_EsmPathList.GetHeadPosition();

	while(posFiles != NULL)
	{
		dialOp.AddEsmFile(m_EsmFilesList.GetNext(posFiles),m_EsmPathList.GetNext(posPaths));
	}

	dialOp.AddRepairPlugin(m_strDlgEspPath + m_strDlgEspFile);

	dialOp.Execute(m_bWorkerThreadForceExit);

}


void CTESPCDDoc::BeginEditor()
{
	CPtrArray * pDataArray, * pEntryPtrArray;
	int tcnt, i,j, nCurRow, cArray, cEntryPtrArray;
	CEditorView* pEView;
	GV_ITEM Item;
	CMapStringToOb * pTempMap;
	POSITION posMap;
	CString mapKey, * pstrFilename;
	CTESEntry * pEntry;
	CTESOperations tesOp;
	CString barstr;

	if(m_iViewDataMode == ESPVIEW)
	{
		pDataArray = &m_aEspDataArray;
		tcnt = m_aEsmDataArray.GetSize() ;

		for(i=tcnt-1;i>=0;i--)
		{
			tesOp.AddEsm(&((CTESFile*)m_aEsmDataArray.GetAt(i))->m_EntryMap);
		}

	}
	else if(m_iViewDataMode == ESMVIEW)
	{
		pDataArray = &m_aEsmDataArray;
	}

	if(!m_bWorkerThreadForceExit)
	{
		Item.mask = GVIF_TEXT|GVIF_FORMAT;
		Item.nFormat = DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS|DT_NOPREFIX;

		pEView = (CEditorView *) GetView("CEditorView");

		pEView->m_pGridCtrl->DeleteNonFixedRows();

		cArray = pDataArray->GetSize();

		for(tcnt = 0, i = 0;i < cArray  && !m_bWorkerThreadForceExit;i++)
		{
			pTempMap = &((CTESFile*)pDataArray->GetAt(i))->m_EntryMap;

			if(m_strEditorFile == "" || m_strEditorFile == *(CString *) (*pTempMap)["FILE"])
			{
				posMap = pTempMap->GetStartPosition();
				
				while(posMap != NULL && !m_bWorkerThreadForceExit)
				{
					pTempMap->GetNextAssoc(posMap,mapKey,(CObject*&)pEntryPtrArray);	

					cEntryPtrArray = pEntryPtrArray->GetSize();
					
					for(j = 0;j < cEntryPtrArray && 
							  mapKey != "PATH" && mapKey != "FILE" &&
							  !m_bWorkerThreadForceExit; j++, tcnt++);
					
				}
			}
		}

		nCurRow = 1;
		
		pEView->m_pGridCtrl->SetRowCount(tcnt + 1);

		cArray = pDataArray->GetSize();

		barstr = "Setting up Editor - %d%% complete";
		if(!m_bWorkerThreadForceExit)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)tcnt);

		for(i = 0;i < cArray  && !m_bWorkerThreadForceExit;i++)
		{
			pTempMap = &((CTESFile*)pDataArray->GetAt(i))->m_EntryMap;

			pstrFilename = (CString *) (*pTempMap)["FILE"];

			if(m_strEditorFile == "" || m_strEditorFile == *pstrFilename)
			{

				posMap = pTempMap->GetStartPosition();
				
				while(posMap != NULL && !m_bWorkerThreadForceExit)
				{
					pTempMap->GetNextAssoc(posMap,mapKey,(CObject*&)pEntryPtrArray);	
					

					cEntryPtrArray = pEntryPtrArray->GetSize();
					
					for(j = 0;j < cEntryPtrArray && 
							  mapKey != "PATH" && mapKey != "FILE" &&
							  !m_bWorkerThreadForceExit; j++, nCurRow++)
					{
						pEntry = (CTESEntry*)pEntryPtrArray->GetAt(j);					

						Item.row = nCurRow;
						
						Item.col = 0;
						Item.strText = mapKey;
						pEView->m_pGridCtrl->SetItem(&Item);

						Item.col = 1;
						Item.strText = tesOp.GetDesc(mapKey);
						pEView->m_pGridCtrl->SetItem(&Item);

						Item.col = 2;
						Item.strText = pEntry->EntryName;
						pEView->m_pGridCtrl->SetItem(&Item);
						
						Item.col = 3;
						if(m_iViewDataMode == ESPVIEW)
						{
							Item.strText = *pstrFilename;
						    pEView->m_pGridCtrl->SetItemData(nCurRow,3,pEntry->EntryStartPos);
						}
						else
						{
							Item.strText = "None";
						}
						pEView->m_pGridCtrl->SetItem(&Item);

						Item.col = 4;
						if(m_iViewDataMode == ESPVIEW)
						{
							Item.strText = tesOp.GetEsmOrigin(m_bWorkerThreadForceExit,pEntry->EntryName,mapKey);
						}
						else if(m_iViewDataMode == ESMVIEW)
						{
							Item.strText = *pstrFilename;
							pEView->m_pGridCtrl->SetItemData(nCurRow,4,pEntry->EntryStartPos);
						}
						else
						{
							Item.strText = "None";
						}
						pEView->m_pGridCtrl->SetItem(&Item);

						barstr.Format("Setting up Editor - %d%% complete", nCurRow*100/tcnt);
						if(!m_bWorkerThreadForceExit)
							SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,(UINT)tcnt);

					}
				}
			}
		}

		if(m_iViewDataMode == ESPVIEW)
		{
			pEView->m_pGridCtrl->SortItems(pEView->m_pGridCtrl->pfnItemDataNumericCompare,3, TRUE);
		}
		else if(m_iViewDataMode == ESMVIEW)
		{
			pEView->m_pGridCtrl->SortItems(pEView->m_pGridCtrl->pfnItemDataNumericCompare,4, TRUE);
		}
		else
		{
			pEView->m_pGridCtrl->SortItems(0, TRUE);
		}

		pEView->m_pGridCtrl->Refresh();

		if(!m_bWorkerThreadForceExit)
		{
			//Destroy the Bar
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT)&barstr,0);
		}

		if(tcnt > 0)
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELON,3,0);
		else
			SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,3,0);
	}

}


void CTESPCDDoc::OnFileNew() 
{
	
	int tcnt,i;
	CTreeView* pView;
	CTreeCtrl* pTreeCtrl;

	pView = (CTreeView*)GetView("CLeftView");
	pTreeCtrl = &pView->GetTreeCtrl();

	// Force any active operation off
	m_bWorkerThreadForceExit = true;
	WaitForSingleObject(m_hWorkerThread,5000);
	SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_STEPPBAR,(UINT) NULL,0);
	m_bWorkerThreadForceExit = false;

	SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,0,0);

	m_EspFilesList.RemoveAll();
	m_EspPathList.RemoveAll();
	m_EspExtractedFileList.RemoveAll();
	m_EspExtractedPathList.RemoveAll();
	m_EspMasterName = "";
	m_EspMasterPath = "";
	m_EspMasterExtractedName = "";
	m_EspMasterExtractedPath = "";

	((CConflictView *)GetView("CConflictView"))->m_pGridCtrl->DeleteNonFixedRows();
	((CWarningView *)GetView("CWarningView"))->m_pGridCtrl->DeleteNonFixedRows();
	((CEditorView *)GetView("CEditorView"))->m_pGridCtrl->DeleteNonFixedRows();

	((CConflictView *)GetView("CConflictView"))->m_pGridCtrl->Refresh();
	((CWarningView *)GetView("CWarningView"))->m_pGridCtrl->Refresh();
	((CEditorView *)GetView("CEditorView"))->m_pGridCtrl->Refresh();

	if(m_FilesRoot)
	{	//Forces Deletion of any Leafs
		pTreeCtrl->DeleteItem(m_FilesRoot);
		m_FilesRoot = pTreeCtrl->InsertItem("Plugin Files",1,1,TVI_ROOT, TVI_LAST);
	}

	if(m_SingleMaster)
	{	//Forces Deletion of any Leafs
		pTreeCtrl->DeleteItem(m_SingleMaster);
		m_SingleMaster = pTreeCtrl->InsertItem("Master Plugin",1,1,TVI_ROOT, TVI_FIRST);
	}

	tcnt = m_aEspDataArray.GetSize();
	for(i = 0; i < tcnt;i++)
	{
		delete ((CTESFile*)m_aEspDataArray.GetAt(i));
	}

	m_aEspDataArray.RemoveAll();

}

void CTESPCDDoc::OnContextadd() 
{
	
	CTreeView* pView;
	CTreeCtrl* pTreeCtrl;
	HTREEITEM treeitem;
	CString itemtxt, parenttxt;

	pView = (CTreeView*)GetView("CLeftView");
	pTreeCtrl = &pView->GetTreeCtrl();

	treeitem = pTreeCtrl->GetSelectedItem();
	if(treeitem == NULL)
	{
		OnFileChooseplugins();
		return;
	}

	itemtxt = pTreeCtrl->GetItemText(treeitem);
	parenttxt = pTreeCtrl->GetItemText(pTreeCtrl->GetParentItem(treeitem));

	//If Selected Child or Parent Call respective Open Command
	if(itemtxt == "Plugin Files" || parenttxt == "Plugin Files")
	{
		OnFileChooseplugins();
	}
	else if(itemtxt == "Master Plugin" || parenttxt == "Master Plugin")
	{
		OnFileChoosemaster();
	}

}

void CTESPCDDoc::OnContextremove() 
{
	
	CTreeView* pView;
	CTreeCtrl* pTreeCtrl;
	CString itemtxt, listtxt;
	POSITION pos1,pos2,pos3;
	int i, tcnt;
	HTREEITEM treeitem;


	pView = (CTreeView*)GetView("CLeftView");
	pTreeCtrl = &pView->GetTreeCtrl();

	treeitem = pTreeCtrl->GetSelectedItem();
	if(treeitem == NULL)
		return;
	itemtxt = pTreeCtrl->GetItemText(treeitem);

	if(itemtxt != "Plugin Files" && itemtxt != "Master Plugin" &&
		itemtxt != m_EspMasterName && itemtxt != m_EspMasterExtractedName)
	{

		pos1 = m_EspFilesList.GetHeadPosition(); 
		pos2 = m_EspPathList.GetHeadPosition();

		if(m_EspFilesList.GetCount() != 0)
		{
			do
			{ //Check m_EspFilesList for Item
				m_EspPathList.GetNext(pos2);
				listtxt = m_EspFilesList.GetNext(pos1);
				if(itemtxt == listtxt)
				{
					if(pos1 == NULL)
					{
						m_EspFilesList.RemoveTail();
						m_EspPathList.RemoveTail();
					}
					else
					{
						m_EspFilesList.GetPrev(pos1);
						m_EspPathList.GetPrev(pos2);
						m_EspFilesList.RemoveAt(pos1);
						m_EspPathList.RemoveAt(pos2);
					}
					pTreeCtrl->DeleteItem(treeitem);
				}
			}while(pos1 != NULL && itemtxt != listtxt);
		}

		if(pos1 == NULL && itemtxt != listtxt && m_EspExtractedFileList.GetCount() > 0)
		{ //If not Contained in m_EspFilesList, Check m_EspExtractedFileList
			for(i=0, pos3 = m_EspExtractedFileList.GetHeadPosition();
				pos3 != NULL && itemtxt != listtxt;
				i++)
			{
				listtxt = m_EspExtractedFileList.GetNext(pos3);
				if(itemtxt == listtxt)
				{
					if(pos3 == NULL)
					{
						m_EspExtractedFileList.RemoveTail();
					}
					else
					{
						m_EspExtractedFileList.GetPrev(pos3);
						m_EspExtractedFileList.RemoveAt(pos3);
					}

					delete ((CTESFile*)m_aEspDataArray.GetAt(i));
					m_aEspDataArray.RemoveAt(i);

					pTreeCtrl->DeleteItem(treeitem);
				}
			}
		}
	}
	else if(itemtxt == m_EspMasterName)
	{
		m_EspMasterName = "";
		m_EspMasterPath = "";
		pTreeCtrl->DeleteItem(treeitem);
	}
	else if(itemtxt == m_EspMasterExtractedName)
	{
		m_EspExtractedFileList.RemoveTail();
		tcnt = m_aEspDataArray.GetSize();
		delete ((CTESFile*)m_aEspDataArray.GetAt(tcnt - 1));
		m_aEspDataArray.RemoveAt(tcnt - 1);
		m_EspMasterExtractedName = "";
		pTreeCtrl->DeleteItem(treeitem);
	}
}

void CTESPCDDoc::OnOperationsOptions() 
{
	
	CPropertySheet PSheet(ID_OPTIONS,AfxGetMainWnd(),0);
	PSheet.m_psh.dwFlags |= PSH_NOAPPLYNOW;

	CPPageID ppageID;
	CPPageOP ppageOP;

	PSheet.AddPage(&ppageOP);
	PSheet.AddPage(&ppageID);
	
	PSheet.DoModal();
}


/*	-------------------------------------------------------------------------------------------
	Name: BOOL GetEntry(CString EspFilename, CString strEntryName, CString strEntryType, unsigned char *&pBuf, int &len)

	Desc: Sets the pointer pBuf to the Data Dump of the Entry specified by the First Three Parameters

	Ret: TRUE if found Data Entry, else FALSE
	-------------------------------------------------------------------------------------------	*/
BOOL CTESPCDDoc::GetEntry(const CString strFilename, const CString strEntryName,
						  const CString strEntryType, CTESEntry *&pEntry,
						  const BOOL bFromEsp)
{
	CMapStringToOb *tempMap;
	CPtrArray *pPtrList;
	CString strName, *pstrFilename;
	int cPtrList;
	CPtrArray *pDataArray;

	if(bFromEsp)
	{
		pDataArray = &m_aEspDataArray;
	}
	else
	{
		pDataArray = &m_aEsmDataArray;
	}


	int arraysize = pDataArray->GetSize();

	for(int i = 0;i < arraysize;i++)
	{

		tempMap = &((CTESFile*)pDataArray->GetAt(i))->m_EntryMap;

		pstrFilename = ((CString*)(*tempMap)["FILE"]);
		if(*pstrFilename == strFilename)
		{
			tempMap->Lookup(strEntryType,(CObject*&)pPtrList);
			
			cPtrList = pPtrList->GetSize();
			for(int j = 0;j < cPtrList; j++)
			{
				strName = ((CTESEntry*)pPtrList->GetAt(j))->EntryName;

				if(strName == strEntryName)
				{
					pEntry = (CTESEntry*)pPtrList->GetAt(j);
					return true;
				}
			}
		}
	}
	
	return false;
}

BOOL CTESPCDDoc::GetTesFileObj(const CString strFilename, CTESFile *&pTesFile, 
							   const BOOL bFromEsp)
{
	CMapStringToOb *tempMap;
	CString *pstrFilename;
	CPtrArray *pDataArray;
	int arraysize;

	if(bFromEsp)
	{
		pDataArray = &m_aEspDataArray;
	}
	else
	{
		pDataArray = &m_aEsmDataArray;
	}

	arraysize = pDataArray->GetSize();

	for(int i = 0;i < arraysize;i++)
	{

		tempMap = &((CTESFile*)pDataArray->GetAt(i))->m_EntryMap;

		pstrFilename = ((CString*)(*tempMap)["FILE"]);
		if(*pstrFilename == strFilename)
		{
			pTesFile = (CTESFile*)pDataArray->GetAt(i);
			return true;
		}
	}
	
	return false;
}


BOOL CTESPCDDoc::UpdateEspEntry(const CStringArray *pID, const CStringArray *pData, 
								const CString EntryID, const CString EntryName, 
								const CString EspFileName)
{

	CTESFile *pEspFile;

	if(GetTesFileObj(EspFileName,pEspFile,true))
	{

		CTESEntry *pUpdateEntry;

		if(GetEntry(EspFileName,EntryName,EntryID,pUpdateEntry,true))
		{
			if(MessageBox(AfxGetMainWnd()->GetSafeHwnd(),
				"Update Entry?\nBackup Saved with Extension .OLD", 
				"Update Entry",MB_YESNO | MB_ICONQUESTION) == IDYES)
			{
				if(!pEspFile->UpdateEntry(pUpdateEntry, pID, pData))
				{
					// Change Unnecessary or Error?
				}
			}
		}
		else
		{
			AfxMessageBox("Entry Deleted or Missing.",MB_OK);
		}
	}
	else
	{
		AfxMessageBox("Filename not Found.",MB_OK);
	}

	return true;
}

BOOL CTESPCDDoc::RemoveEspEntry(const CString strFilename, const CString ID, 
								const CString EntryName, const BOOL ShowQuestion)
{

	CTESFile *pEspFile;

	if(GetTesFileObj(strFilename,pEspFile,true))
	{

		CTESEntry *pDeleteEntry;

		if(GetEntry(strFilename,EntryName,ID,pDeleteEntry,true))
		{
			if(!ShowQuestion || MessageBox(AfxGetMainWnd()->GetSafeHwnd(),
				"Delete Entry(s)?\n\nBackup(s) Saved with Extension \".OLD\"",
				"Delete Entry(s)",MB_YESNO | MB_ICONQUESTION) == IDYES)
			{
				if(pEspFile->DeleteEntry(*pDeleteEntry))
				{
					pEspFile->RemoveEntry(pDeleteEntry,ID);
					return true;
				}
			}
		}
		else
		{
			AfxMessageBox("Entry Already Deleted or Missing.",MB_OK);
		}
	}
	else
	{
		AfxMessageBox("Filename not Found.",MB_OK);
	}
	
	return false;
}

BOOL CTESPCDDoc::IsWorkerThreadActive()
{
	return m_bWorkerThreadActive;
}

BOOL CTESPCDDoc::IsWorkerForcedExit()
{
	return m_bWorkerThreadForceExit;
}

void CTESPCDDoc::SetWorkerThreadForced(BOOL bForceExit)
{
	m_bWorkerThreadForceExit = bForceExit;
}

HANDLE CTESPCDDoc::GetWorkerThread()
{
	return m_hWorkerThread;
}

void CTESPCDDoc::SetLoadEsms(BOOL bDoLoad)
{
	m_bLoadEsms = bDoLoad;
}

BOOL CTESPCDDoc::GetLoadEsms()
{
	return m_bLoadEsms;
}

void CTESPCDDoc::OnOperationsClearesmdata() 
{
	int tcnt = m_aEsmDataArray.GetSize();
	for(int i = 0; i < tcnt;i++)
	{
		delete ((CTESFile*)m_aEsmDataArray.GetAt(i));
	}

	m_aEsmDataArray.RemoveAll();

	m_EsmFilesList.RemoveAll();
	m_EsmPathList.RemoveAll();

	m_bEsmExtracted = FALSE;

	SendMessage(AfxGetMainWnd()->GetSafeHwnd(),UDM_PANELOFF,0,0);

	((CConflictView *)GetView("CConflictView"))->m_pGridCtrl->DeleteNonFixedRows();
	((CWarningView *)GetView("CWarningView"))->m_pGridCtrl->DeleteNonFixedRows();
	((CEditorView *)GetView("CEditorView"))->m_pGridCtrl->DeleteNonFixedRows();

	((CConflictView *)GetView("CConflictView"))->m_pGridCtrl->Refresh();
	((CWarningView *)GetView("CWarningView"))->m_pGridCtrl->Refresh();
	((CEditorView *)GetView("CEditorView"))->m_pGridCtrl->Refresh();

}
