/////////////////////////////////////////////////////////////////////////////
// TESPCDDoc.h : header file
//
// Written by Ely Soto <elyvsoto@hotmail.com>
// Copyright (c) 2002. All Rights Reserved.
//
// This code may be used in compiled form in any way you desire. This software
// may not be sold for profit without the authors written consent.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
//////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_TESPCDDOC_H__ABE7A161_41C0_4F97_8141_D03209317383__INCLUDED_)
#define AFX_TESPCDDOC_H__ABE7A161_41C0_4F97_8141_D03209317383__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define TESDEFPATH "C:\\Program Files\\Bethesda Softworks\\Morrowind\\Data Files\\\0"
#define OPENDIALOG_BUFFERSIZE 10000

#include "TESOperations.h"
#include "TESFile.h"

class CTESPCDDoc : public CDocument
{
protected: // create from serialization only
	CTESPCDDoc();
	DECLARE_DYNCREATE(CTESPCDDoc)

// Attributes

public:
	enum _OP_TYPE
	{
		FULLCHECK,
		SINGLEVERSUS
	} m_nOperationType;

	// Conflict Types Checked
	CTESOperations::_sOptionTypes m_ConflictCheckTypes;
	CTESOperations::_sOptionTypes m_UncleanCheckTypes;

	// Extraction Options Struct
	CTESFile::_sExtractOpt m_ExtractOpt;

	// Default Operations
	UCHAR m_DefOpFlags;

	// Tree Root Items in TreeView
	HTREEITEM m_FilesRoot;
	HTREEITEM m_SingleMaster;

	// Morrowind Program Paths
	CString m_strDataFilesPath;
	CString m_strBasePath;

	// Find Object History Strings
	CString m_strFind[5];
	UCHAR m_cFindHistory;

	// Force Enables Dialog Relinker
	BOOL m_bEnableDialogRelink;

protected:
	// Strings and Lists of Esp Filenames and Paths
	CString m_EspMasterPath;
	CString m_EspMasterName;
	CString m_EspMasterExtractedName;
	CString m_EspMasterExtractedPath;
	CStringList m_EspPathList;
	CStringList m_EspFilesList;
	CStringList m_EspExtractedFileList;
	CStringList m_EspExtractedPathList;

	// Array of pointers to each Extracted Esp File
	CPtrArray m_aEspDataArray;
	
	// Esm File and Path Strings and Extraction State
	BOOL m_bEsmExtracted;
	CStringList m_EsmPathList;
	CStringList m_EsmFilesList;	
	
	// Array of pointers to each Extracted Esm File
	CPtrArray m_aEsmDataArray;

	// Current Thread Operation Type and Pointer
	int m_nThreadOperation;
	CWinThread* m_pWorkerThread;
	// Thread Status and Control
	BOOL m_bWorkerThreadForceExit;
	BOOL m_bWorkerThreadActive;
	// Worker Thread Handle
	HANDLE m_hWorkerThread;

	// Operation Flags
	UCHAR m_CurOpFlags;

	// Load Esm files for comparisons
	BOOL m_bLoadEsms;

	// Global Changes Variables
	CString m_strGlobCheckFile;

	// Editor View Variables
	CString m_strEditorFile;
	enum _VIEW_TYPE
	{
		ESPVIEW,
		ESMVIEW
	} m_iViewDataMode;

	// Find Dialog Variables
	BOOL m_bFOMatchcase;
	BOOL m_bFOMatchword;
	BOOL m_bFOLookdata;
	BOOL m_bFOSearchEsm;
	CString m_strFOObjName;

	// Repair Tool Variables
	BOOL m_bRepairAuto;
	CString m_strDlgEspFile;
	CString m_strDlgEspPath;

public:
// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTESPCDDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	//}}AFX_VIRTUAL

// Implementation
public:
	BOOL GetLoadEsms();
	void SetLoadEsms(BOOL bDoLoad);
	HANDLE GetWorkerThread();	// Returns Handle to Worker Thread
	void SetWorkerThreadForced(const BOOL bForceExit);	// Sets Exit State of Thread
	BOOL IsWorkerForcedExit();	// Checks Exit State of Thread
	BOOL IsWorkerThreadActive();
	
	// Returns View this Frame Owns or NULL if classname not Found
	CView* GetView(const CString classname);
	
	BOOL RemoveEspEntry(const CString strFilename, const CString ID, 
						const CString EntryName, const BOOL ShowQuestion = TRUE);
	
	// Updates Entry with Parsed Data in pID and pData
	BOOL UpdateEspEntry(const CStringArray *pID, const CStringArray *pData, 
						const CString EntryID, const CString EntryName, 
						const CString EspFileName);
	
	// Worker Thread Start Function
	friend unsigned int ThreadFunc(LPVOID);
	
	// Retrieves an Entry from Local Array
	BOOL GetEntry(const CString Filename, const CString strEntryName,
				  const CString strEntryType, CTESEntry *&pEntry, 
				  const BOOL bFromEsp);
	
	// Retrieves Entire CTESFile Object from Local Array
	BOOL GetTesFileObj(const CString EspFilename, CTESFile *&pEspFile, const BOOL bFromEsp);

	virtual ~CTESPCDDoc();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

// Generated message map functions
protected:
	void BeginDialogRepair();
	void BeginEditor();
	void BeginFindGlobalChanges();
	void BeginFind();
	void PrepareCheck();
	void BeginAutoClean();
	void BeginExtract();
	void BeginCheck();

	BOOL GetESMPaths();	// Retrieve Paths to Esm Master Files
	
	//{{AFX_MSG(CTESPCDDoc)
	afx_msg void OnFileChooseplugins();
	afx_msg void OnFileChoosemaster();
	afx_msg void OnUpdateFileChoosemaster(CCmdUI* pCmdUI);
	afx_msg void OnFileSave();
	afx_msg void OnOperationsExtractdata();
	afx_msg void OnOperationsCheckconflicts();
	afx_msg void OnFileNew();
	afx_msg void OnContextadd();
	afx_msg void OnContextremove();
	afx_msg void OnOperationsOptions();
	afx_msg void OnFileChooseactiveplugins();
	afx_msg void OnOperationsAutoclean();
	afx_msg void OnOperationsCheckall();
	afx_msg void OnOperationsCheckdeleted();
	afx_msg void OnOperationsCheckunclean();
	afx_msg void OnOperationsCheckAllGlobalChanges();
	afx_msg void OnOperationsFindobject();
	afx_msg void OnOperationsRepairdialog();
	afx_msg void OnOperationsAutorepairdialog();
	afx_msg void OnOperationsViewallespdata();
	afx_msg void OnOperationsViewallesmdata();
	afx_msg void OnOperationsViewfiledata();
	afx_msg void OnOperationsClearesmdata();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TESPCDDOC_H__ABE7A161_41C0_4F97_8141_D03209317383__INCLUDED_)
